package com.kingtom.kirin.api.controller;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.dev33.satoken.annotation.SaMode;
import com.github.pagehelper.PageInfo;
import com.kingtom.kirin.api.auth.dto.AuthMenuDTO;
import com.kingtom.kirin.api.auth.dto.AuthMenuTreeDTO;
import com.kingtom.kirin.api.auth.dto.AuthRoleDTO;
import com.kingtom.kirin.api.auth.mapper.AuthMapper;
import com.kingtom.kirin.api.dto.IdsDTO;
import com.kingtom.kirin.api.security.SaSecurityUtils;
import com.kingtom.kirin.app.auth.model.AuthMenu;
import com.kingtom.kirin.app.auth.model.AuthRole;
import com.kingtom.kirin.app.auth.service.IAuthService;
import com.kingtom.kirin.app.system.utils.SystemUtils;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.base.Result;
import com.kingtom.kirin.core.common.exception.AppException;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.common.utils.JsonUtils;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/8/29
 */

@Tag(name = "AuthController", description = "角色、菜单、权限相关接口")
@RequestMapping("/auth")
@RestController
public class AuthController{

    @Autowired
    private IAuthService authService;

    @Operation(summary = "角色分页查询")
    @Parameters({@Parameter(name = "query", description = "搜索内容", in = ParameterIn.QUERY, required = false),
            @Parameter(name = "pageNum", description = "当前页", in = ParameterIn.QUERY, required = true),
            @Parameter(name = "pageSize", description = "分页", in = ParameterIn.QUERY, required = true)})
    @GetMapping("/auth_role/list")
    @SaCheckPermission(value = {"AuthMenu:perSetting_role:Execute"})
    public Result<PageInfo<AuthRoleDTO>> getRoleList(String query, int pageNum, int pageSize){
        String search = null;
        if(StringUtils.isNotBlank(query)){
            Map<String, String> map = JsonUtils.toObject(query, Map.class);
            search = map.get("search");
        }
        PageInfo<AuthRole> page = authService.findRolePage(search, pageNum, pageSize);
        PageInfo<AuthRoleDTO> r = CollectionUtils.convertPage(page, AuthMapper.INSTANCT::convert);
        return Result.ok(r);
    }

    @Operation(summary = "新建角色")
    @PostMapping("/auth_roles/")
    @SaCheckPermission(value = {"AuthMenu:perSetting_role:Execute"})
    public Result<AuthRoleDTO> addRole(@RequestBody AuthRoleDTO dto){
        AuthRole role = AuthMapper.INSTANCT.convert(dto);
        role.setSysId(SystemUtils.getCurrentSystemId());
        role.setDisplay(CommonConst.BooleanEnum.True.getKey());
        role = authService.addRole(role, dto.getMenuIds());
        if(role == null){
            Result.ok(null);
        }
        return Result.ok(AuthMapper.INSTANCT.convert(role));
    }

    @Parameter(name = "id", description = "角色Id", in = ParameterIn.PATH, required = true)
    @Operation(summary = "删除角色")
    @PostMapping("/auth_roles/{id}/")
    @SaCheckPermission(value = {"AuthMenu:perSetting_role:Execute"})
    public Result<String> deleteRole(@PathVariable("id") String id){
        authService.deleteRoleById(id);
        return Result.ok("操作成功！");
    }

    @Operation(summary = "批量删除角色")
    @PostMapping("/auth_roles/delete/")
    @SaCheckPermission(value = {"AuthMenu:perSetting_role:Execute"})
    public Result<String> deleteRoles(@RequestBody IdsDTO ids){
        authService.deleteRoleByIds(ids.getIds());
        return Result.ok("操作成功！");
    }

    @Operation(summary = "更新角色")
    @PostMapping("/auth_roles/{id}/info")
    @SaCheckPermission(value = {"AuthMenu:perSetting_role:Execute"})
    public Result<AuthRoleDTO> updateRole(@PathVariable("id") String id, @RequestBody AuthRoleDTO dto){
        AuthRole role = AuthMapper.INSTANCT.convert(dto);
        role.setId(id);
        role = authService.updateRole(role, dto.getMenuIds());
        if(role == null){
            Result.ok(null);
        }
        return Result.ok(AuthMapper.INSTANCT.convert(role));
    }

    @Parameter(name = "id", description = "角色Id", in = ParameterIn.PATH, required = true)
    @Operation(summary = "根据id查找角色")
    @GetMapping("/auth_roles/{id}/info")
    @SaCheckPermission(value = {"AuthMenu:perSetting_role:Execute"})
    public Result<AuthRoleDTO> findRole(@PathVariable("id") String id){
        AuthRole role = authService.findRoleById(id);
        if(role == null){
            Result.ok(null);
        }
        List<String> menuIds = authService.findBindMenuByRoleId(id);
        AuthRoleDTO dto = AuthMapper.INSTANCT.convert(role);
        dto.setMenuIds(menuIds);
        return Result.ok(dto);
    }

    @Operation(summary = "设计态：菜单树列表")
    @GetMapping("/auth_menus/list")
    @SaCheckPermission(value = {"AuthMenu:perSetting_menu:Execute", "AuthMenu:perSetting_role:Execute"},
            mode = SaMode.OR)
    public Result<List<AuthMenuTreeDTO>> getMenuList(){
        List<AuthMenu> list = authService.findMenuListBySystem(null, SystemUtils.getCurrentSystemId());
        List<AuthMenuDTO> dtoList = list.stream().map(AuthMapper.INSTANCT::convert).collect(Collectors.toList());
        return Result.ok(AuthMenuTreeDTO.build(dtoList));
    }

    @Operation(summary = "新建菜单")
    @PostMapping("/auth_menus/")
    @SaCheckPermission(value = {"AuthMenu:perSetting_menu:Execute"})
    public Result<AuthMenuDTO> addMenu(@RequestBody AuthMenuDTO dto){
        AuthMenu menu = AuthMapper.INSTANCT.convert(dto);
        menu.setPreset(CommonConst.BooleanEnum.False.getKey());
        menu.setSysId(SystemUtils.getCurrentSystemId());
        menu = authService.addMenu(menu);
        if(menu == null){
            Result.ok(null);
        }
        AuthMenuDTO dto1 = AuthMapper.INSTANCT.convert(menu);
        AuthMenu parentMenu = authService.findMenuById(menu.getParentId());
        dto1.setParentName(parentMenu == null ? null : parentMenu.getName());
        return Result.ok(dto1);
    }

    @Parameter(name = "id", description = "菜单Id", in = ParameterIn.PATH, required = true)
    @Operation(summary = "删除菜单")
    @PostMapping("/auth_menus/{id}/")
    @SaCheckPermission(value = {"AuthMenu:perSetting_menu:Execute"})
    public Result<String> deleteMenu(@PathVariable("id") String id){
        //接口控制预置菜案不能删除
        AuthMenu menu = authService.findMenuById(id);
        if(CommonConst.BooleanEnum.True.getKey().equals(menu.getPreset())){
            throw new AppException("预置菜单不能删除");
        }
        authService.deleteMenuById(id, true);
        return Result.ok("操作成功！");
    }

    //    @Operation(summary = "批量删除菜单")
    //    @PostMapping("/auth_menus/delete/")
    //    public Result<String> deleteMenus(@RequestBody IdsDTO ids){
    //        authorityService.deleteMenuByIds(ids.getIds());
    //        return Result.ok("操作成功！");
    //    }

    @Operation(summary = "更新菜单")
    @PostMapping("/auth_menus/{id}/info")
    @SaCheckPermission(value = {"AuthMenu:perSetting_menu:Execute"})
    public Result<AuthMenuDTO> updateMenu(@PathVariable("id") String id, @RequestBody AuthMenuDTO dto){
        AuthMenu menu = AuthMapper.INSTANCT.convert(dto);
        menu.setId(id);
        menu = authService.updateMenu(menu);
        if(menu == null){
            Result.ok(null);
        }
        AuthMenuDTO dto1 = AuthMapper.INSTANCT.convert(menu);
        AuthMenu parentMenu = authService.findMenuById(menu.getParentId());
        dto1.setParentName(parentMenu == null ? null : parentMenu.getName());
        return Result.ok(dto1);
    }

    @Parameter(name = "id", description = "", in = ParameterIn.PATH, required = true)
    @Operation(summary = "查看菜单信息")
    @GetMapping("/auth_menus/{id}/info")
    @SaCheckPermission(value = {"AuthMenu:perSetting_menu:Execute"})
    public Result<AuthMenuDTO> findMenu(@PathVariable("id") String id){
        AuthMenu menu = authService.findMenuById(id);
        if(menu == null){
            Result.ok(null);
        }
        return Result.ok(AuthMapper.INSTANCT.convert(menu));
    }

    @Operation(summary = "运行态：菜单树列表")
    @GetMapping("/auth_menus/run_list")
    public Result<List<AuthMenuTreeDTO>> getMenuListRuntime(){
        List<AuthMenu> list = authService.findMenuListBySystemRuntime(SaSecurityUtils.findSessionData().getUserId(),
                SystemUtils.getCurrentSystemId());
        List<AuthMenuDTO> dtoList = list.stream().map(AuthMapper.INSTANCT::convert).collect(Collectors.toList());
        return Result.ok(AuthMenuTreeDTO.build(dtoList));
    }

}
