package com.kingtom.kirin.app.auth.base;

import java.util.*;
import java.util.stream.Collectors;

import com.kingtom.kirin.app.auth.AuthConst;
import com.kingtom.kirin.app.auth.model.AuthMenu;
import com.kingtom.kirin.app.auth.model.AuthRole;
import com.kingtom.kirin.app.auth.model.AuthRoleSource;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.json.JObject;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.common.utils.IDUtils;
import com.kingtom.kirin.core.resource.ResourceObject;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.config.BeanPostProcessor;
import org.springframework.core.annotation.AnnotationUtils;
import org.springframework.stereotype.Component;

/**
 * 权限模块</p>
 * 菜单初始化</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023 /9/2
 */
@Component
public class AuthInitProcessor implements BeanPostProcessor{

    private Map<String, Menu> menus = new HashMap<>();

    private Map<String, Role> roles = new HashMap<>();

    private Map<String, List<String>> roleMenuMap = new HashMap<>();


    @Override
    public Object postProcessBeforeInitialization(Object bean, String beanName) throws BeansException{
        AuthInit init = AnnotationUtils.getAnnotation(bean.getClass(), AuthInit.class);
        if(init == null){
            return bean;
        }
        for(Menu menu : init.menus()){
            menus.put(menu.name(), menu);
        }
        for(Role role : init.roles()){
            roles.put(role.code(), role);
        }
        return bean;
    }

    /**
     * Create menu bean list.
     *
     * @param sysId the sys id
     * @return the list
     */
    public List<AuthMenu> createMenuBean(String sysId){
        Map<String, AuthMenu> map = new HashMap<>();
        List<AuthMenu> list = new ArrayList<>();
        List<Menu> menuCollection = new ArrayList<>(menus.values());
        menuCollection.sort(Comparator.comparingInt(a -> a.name().length()));

        for(Menu m : menuCollection){
            String parentId = null;
            String name = m.name();
            String fullName = m.name();
            if(m.name().contains(AuthConst.MENU_NAME_SEPARATOR)){
                int index = m.name().lastIndexOf(AuthConst.MENU_NAME_SEPARATOR);
                String parentName = m.name().substring(0, index);
                AuthMenu parent = map.get(parentName);
                if(parent == null){
                    //找不到父元素，所以直接跳过
                    continue;
                }
                parentId = parent.getId();
                name = m.name().substring(index + 1);
            }

            AuthMenu menu = new AuthMenu();
            menu.setParentId(parentId);
            menu.setName(name);
            menu.setOtherName(m.otherName());
            menu.setCode(m.code());
            menu.setType(m.type().getKey());
            menu.setHide(m.hide().getKey());
            menu.setPreset(CommonConst.BooleanEnum.True.getKey());
            menu.setActionType(m.actionType().getKey());
            menu.setIcon(m.icon());
            menu.setRouteUri(m.routeUri());
            menu.setOrderNum(m.orderNum());
            menu.setSysId(sysId);
            menu.setId(IDUtils.newUUIDByString(menu.getCode() + menu.getSysId()));
            list.add(menu);
            map.put(fullName, menu);
        }
        return list;
    }

    /**
     * Create role bean list.
     *
     * @param sysId the sys id
     * @return the list
     */
    public List<AuthRole> createRoleBean(String sysId){
        List<AuthRole> list = new ArrayList<>();
        for(Role role : roles.values()){
            AuthRole r = new AuthRole();
            r.setName(role.name());
            r.setCode(role.code());
            r.setSysId(sysId);
            r.setState(CommonConst.StateEnum.Enable.getKey());
            ResourceObject<String> resourceObject = new ResourceObject<>();
            r.setSource(new JObject<>(new AuthRoleSource(new ArrayList<>())));
            r.setDisplay(role.display() == null ? CommonConst.BooleanEnum.True.getKey() : role.display().getKey());
            list.add(r);
        }
        return list;
    }

    /**
     * Find role menu ids list.
     *
     * @param roleCode the role code
     * @param menuList the menu list
     * @return the list
     */
    public List<String> findRoleMenuIds(String roleCode, List<AuthMenu> menuList){
        if(roleMenuMap.isEmpty()){
            initRoleMenuMap();
        }
        List<String> menuCodes = roleMenuMap.get(roleCode);
        if(CollectionUtils.isEmpty(menuCodes)){
            return new ArrayList<>();
        }
        return menuList.stream().filter(e -> menuCodes.contains(e.getCode())).map(AuthMenu::getId)
                .collect(Collectors.toList());
    }

    private void initRoleMenuMap(){
        for(Role role : roles.values()){
            roleMenuMap.put(role.code(), new ArrayList<>());
            List<String> menuCodes = Arrays.asList(role.menuCode());

            if(CollectionUtils.isEmpty(menuCodes)){
                continue;
            }
            if(menuCodes.contains(AuthConst.USER_ROLE_MENU_NONE)){
                continue;
            }
            if(menuCodes.contains(AuthConst.USER_ROLE_MENU_ALL)){
                roleMenuMap.get(role.code())
                        .addAll(menus.values().stream().map(Menu::code).collect(Collectors.toList()));
                continue;
            }
            roleMenuMap.get(role.code()).addAll(menuCodes);

        }

    }

}
