package com.kingtom.kirin.app.auth.listener;

import java.util.*;
import java.util.stream.Collectors;

import com.kingtom.kirin.app.auth.base.AuthInitProcessor;
import com.kingtom.kirin.app.auth.model.AuthMenu;
import com.kingtom.kirin.app.auth.model.AuthRole;
import com.kingtom.kirin.app.auth.service.IAuthService;
import com.kingtom.kirin.app.system.model.SystemInfo;
import com.kingtom.kirin.app.system.service.SystemDefaultService;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.event.ApplicationStartedEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.stereotype.Component;

/**
 * 权限模块</p>
 * 系统用户事件监听器集合</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023 /9/4
 */
@Slf4j
@Component
public class AuthApplicationStartedEventListener implements ApplicationListener<ApplicationStartedEvent>{

    @Autowired
    private SystemDefaultService systemDefaultService;

    @Autowired
    private IAuthService authService;

    @Autowired
    private AuthInitProcessor authInitProcessor;

    @Override
    public void onApplicationEvent(ApplicationStartedEvent event){
        log.info("---开始检查并修复权限---");
        //1、检查各个系统的预置菜单完整性
        SystemInfo systemInfo = systemDefaultService.findDefaultSystemInfo();
        if(systemInfo == null){
            return;
        }
        List<AuthMenu> menus = authService.findMenuListBySystem(CommonConst.BooleanEnum.True, systemInfo.getId());
        List<AuthMenu> newMenuList = authInitProcessor.createMenuBean(systemInfo.getId());
        List<List<AuthMenu>> lists = CollectionUtils.analyzeCollection(menus, newMenuList,
                (a, b) -> menutoString(a).equals(menutoString(b)));
        if(CollectionUtils.isNotEmpty(lists.get(1)) || CollectionUtils.isNotEmpty(lists.get(2))){
            if(CollectionUtils.isNotEmpty(menus)){
                //删除原来的菜单
                authService.deleteMenuByIds(menus.stream().map(AuthMenu::getId).collect(Collectors.toList()));
            }
            //新建新的菜单；
            newMenuList = authService.addMenus(newMenuList);
            log.info("共修复" + newMenuList.size() + "条菜单数据！");
        }
        List<AuthMenu> allMenus = authService.findMenuListBySystem(null, systemInfo.getId());

        //2、检查各个系统的预置角色完整性
        List<AuthRole> allRole = authService.findAllRole(systemInfo.getId());
        Map<String, AuthRole> map = allRole.stream().collect(Collectors.toMap(AuthRole::getCode, e -> e));
        List<AuthRole> roles = authInitProcessor.createRoleBean(systemInfo.getId());
        int i = 0;
        for(AuthRole role : roles){
            AuthRole dbRole = map.get(role.getCode());
            List<String> roleMenuIds = authInitProcessor.findRoleMenuIds(role.getCode(), allMenus);
            if(dbRole == null){
                //创建新角色
                authService.addRole(role, roleMenuIds);
                i++;
            } else{
                //更新角色拥有的菜单权限，只增加菜单权限。不能减少，因为无法判断是否是用户通过界面操作调整的
                List<String> dbBindMenuIds = authService.findBindMenuByRoleId(dbRole.getId());
                HashSet<String> set = new HashSet<>(dbBindMenuIds);
                if(CollectionUtils.isNotEmpty(roleMenuIds) && !set.containsAll(roleMenuIds)){
                    set.addAll(roleMenuIds);
                    dbRole.setCode(role.getCode());
                    dbRole.setName(role.getName());
                    dbRole.setDisplay(role.getDisplay());
                    authService.updateRole(dbRole, new ArrayList<>(set));
                    i++;
                }
            }
        }
        if(i > 0){
            log.info("共修复" + i + "条角色数据！");
        }
        //3、超级管理员角色，绑定所有菜单权限
        AuthRole admin = authService.findSuperAdminRole(systemInfo.getId());
        List<String> menuIds = authService.findBindMenuByRoleId(admin.getId());
        authService.updatePermissionForBindMenu(admin.getId(),
                allMenus.stream().map(AuthMenu::getId).collect(Collectors.toList()), menuIds);
        log.info("---检查并修复权限完成---");

    }

    private String menutoString(AuthMenu menu){
        List<String> array =
                Arrays.asList(menu.getName(), menu.getOtherName(), menu.getCode(), menu.getType(), menu.getActionType(),
                        String.valueOf(menu.getHide()), menu.getIcon(), menu.getRouteUri(),
                        String.valueOf(menu.getOrderNum()));
        return StringUtils.join(array, CommonConst.COMMA_STRING);
    }

}
