package com.kingtom.kirin.app.auth.service;

import java.util.List;

import com.github.pagehelper.PageInfo;
import com.kingtom.kirin.app.auth.model.AuthMenu;
import com.kingtom.kirin.app.auth.model.AuthPermission;
import com.kingtom.kirin.app.auth.model.AuthRole;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.resource.ResourceConst;
import com.kingtom.kirin.core.resource.ResourceObject;

/**
 * 权限模块</p>
 * 身份授权认证 接口</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023 /8/29
 */
public interface IAuthService{

    /**
     * 新建角色 同时绑定人员，绑定菜单
     *
     * @param role    the role
     * @param menuIds the menu ids
     * @return the auth role
     */
    AuthRole addRole(AuthRole role, List<String> menuIds);

    /**
     * 根据Id删除角色，同时删除角色对应的权限
     *
     * @param id the id
     */
    void deleteRoleById(String id);

    /**
     * 根据Id集合删除角色，同时删除角色对应的权限
     *
     * @param ids the ids
     */
    void deleteRoleByIds(List<String> ids);

    /**
     * 更新角色 同时绑定人员，绑定菜单
     *
     * @param role    the role
     * @param menuIds the menu ids
     * @return the auth role
     */
    AuthRole updateRole(AuthRole role, List<String> menuIds);

    /**
     * 角色追加的方式绑定资源
     *
     * @param roleId         the role id
     * @param resourceObject the resource object
     * @return the auth role
     */
    AuthRole updateAuthRoleAddBindResource(String roleId, ResourceObject<String> resourceObject);

    /**
     * 根据id查找角色.
     *
     * @param id the id
     * @return the auth role
     */
    AuthRole findRoleById(String id);

    /**
     * 找到超级管理员角色
     *
     * @param sysId the sys id
     * @return the auth role
     */
    AuthRole findSuperAdminRole(String sysId);

    /**
     * Find user role auth role.
     *
     * @param sysId the sys id
     * @return the auth role
     */
    AuthRole findUserRole(String sysId);

    /**
     * Find all role auth role.
     *
     * @param sysId the sys id
     * @return the auth role
     */
    List<AuthRole> findAllRole(String sysId);

    /**
     * 角色分页查询，不包含超级管理员角色
     *
     * @param search   the search
     * @param pageNum  the page num
     * @param pageSize the page size
     * @return the page info
     */
    PageInfo<AuthRole> findRolePage(String search, int pageNum, int pageSize);

    /**
     * 根据用户id查找角色.
     *
     * @param userId the id
     * @param sysId  the sys id
     * @return the list
     */
    List<AuthRole> findRoleBySystemUser(String userId, String sysId);

    /**
     * 新建菜单，同时给超级管理员绑定权限
     *
     * @param menu the menu
     * @return the auth menu
     */
    AuthMenu addMenu(AuthMenu menu);

    /**
     * 新建菜单，同时给超级管理员绑定权限
     *
     * @param menus the menus
     * @return the list
     */
    List<AuthMenu> addMenus(List<AuthMenu> menus);

    /**
     * 根据id删除菜单，同时解绑权限.
     *
     * @param id          the id
     * @param deleteChild the delete child
     */
    void deleteMenuById(String id, boolean deleteChild);

    /**
     * 根据id集合批量删除菜单，同时解绑权限.
     *
     * @param ids the ids
     */
    void deleteMenuByIds(List<String> ids);

    /**
     * 更新菜单，同时更新权限码.
     *
     * @param menu the menu
     * @return the auth menu
     */
    AuthMenu updateMenu(AuthMenu menu);

    /**
     * 根据id查找菜单.
     *
     * @param id the id
     * @return the auth menu
     */
    AuthMenu findMenuById(String id);

    /**
     * 查找当前系统的菜单.
     *
     * @param isPreset null表示不区分是否预置；
     * @param sysId    the sys id
     * @return the list
     */
    List<AuthMenu> findMenuListBySystem(CommonConst.BooleanEnum isPreset, String sysId);

    /**
     * 运行态查找某用户有权限的菜单集合.
     * 如果有子菜单的权限，没有父菜单的权限，默认会把父菜单一起返回
     *
     * @param userId the user id
     * @param sysId  the sys id
     * @return the list
     */
    List<AuthMenu> findMenuListBySystemRuntime(String userId, String sysId);

    List<AuthMenu> findMenuListByIds(List<String> ids);

    /**
     * 根据角色id查找绑定的菜单id集合.
     *
     * @param id the id
     * @return the list
     */
    List<String> findBindMenuByRoleId(String id);

    /**
     * 更新角色与菜单之间的绑定关系.
     *
     * @param roleId    the role id
     * @param menuIds   the menu ids
     * @param dbMenuIds the db menu ids
     */
    public void updatePermissionForBindMenu(String roleId, List<String> menuIds, List<String> dbMenuIds);

    /**
     * 获取某个用户在系统内的所有权限
     *
     * @param userId the user id
     * @param sysId  the sys id
     * @return auth permissions
     */
    List<AuthPermission> findAllAuthPermissions(String userId, String sysId);

    /**
     * 获取某个用户在系统内的指定类型资源的权限
     *
     * @param resourceTypeEnums the resource type enums
     * @param operaEnums        the opera enums
     * @param userId            the user id
     * @param sysId             the sys id
     * @return the list
     */
    List<AuthPermission> findAuthPermissions(List<ResourceConst.ResourceTypeEnum> resourceTypeEnums,
            List<ResourceConst.ResourceOperaEnum> operaEnums, String userId, String sysId);

}
