package com.kingtom.kirin.app.auth.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import com.kingtom.kirin.app.auth.dao.AuthPermissionMapper;
import com.kingtom.kirin.app.auth.model.AuthPermission;
import com.kingtom.kirin.app.auth.model.AuthPermissionExample;
import com.kingtom.kirin.app.auth.service.IAuthPermissionService;
import com.kingtom.kirin.core.common.exception.AppException;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.common.utils.IDUtils;
import com.kingtom.kirin.core.resource.ResourceConst;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 权限模块</p>
 * 权限 pojo service impl</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/8/24
 */
@Slf4j
@Service
public class AuthPermissionImpl implements IAuthPermissionService{

    @Autowired
    private AuthPermissionMapper mapper;

    @Override
    public AuthPermission create(AuthPermission permission){
        if(permission == null){
            throw new AppException("权限为空，新建权限失败！");
        }
        if(StringUtils.isBlank(permission.getCode())){
            throw new AppException("权限码为空，新建权限失败！");
        }
        if(StringUtils.isBlank(permission.getResType())){
            throw new AppException("资源类型为空，新建权限失败！");
        }
        if(ResourceConst.ResourceTypeEnum.findEnumByKey(permission.getResType()) == null){
            throw new AppException("资源类型错误，新建权限失败！");
        }
        if(StringUtils.isBlank(permission.getResId())){
            throw new AppException("资源id为空，新建权限失败！");
        }
        if(StringUtils.isBlank(permission.getOperaType())){
            throw new AppException("资源操作类型为空，新建权限失败！");
        }
        if(ResourceConst.ResourceOperaEnum.findEnumByKey(permission.getOperaType()) == null){
            throw new AppException("资源操作类型错误，新建权限失败！");
        }
        if(StringUtils.isBlank(permission.getRoleId())){
            throw new AppException("角色id为空，新建权限失败！");
        }
        permission.setId(IDUtils.newUUID());
        permission.setGmtCreate(new Date());
        permission.setGmtModified(permission.getGmtCreate());
        boolean f = mapper.insertSelective(permission) == 1;
        if(!f){
            throw new AppException("落库出错，更新权限失败！");
        }
        return findById(permission.getId());
    }

    @Override
    public List<AuthPermission> createBatch(List<AuthPermission> permission){
        return permission.stream().map(this::create).collect(Collectors.toList());
    }

    @Override
    public AuthPermission update(AuthPermission permission){
        //只允许更新权限码
        if(permission == null){
            throw new AppException("权限为空，更新权限失败！");
        }
        if(StringUtils.isBlank(permission.getId())){
            throw new AppException("权限id为空，更新权限失败！");
        }
        if(StringUtils.isBlank(permission.getCode())){
            throw new AppException("权限码为空，更新权限失败！");
        }
        AuthPermission update = new AuthPermission();
        update.setId(permission.getId());
        update.setCode(permission.getCode());
        update.setGmtModified(new Date());
        boolean f = mapper.updateByPrimaryKeySelective(update) == 1;
        if(!f){
            throw new AppException("落库出错，更新权限失败！");
        }
        return findById(permission.getId());
    }

    @Override
    public AuthPermission findById(String id){
        return mapper.selectByPrimaryKey(id);
    }

    @Override
    public void deleteByRRO(List<ResourceConst.ResourceTypeEnum> resTypeEnums, List<String> resIds,
            List<ResourceConst.ResourceOperaEnum> operaEnums, List<String> roleIds){
        if(CollectionUtils.isEmpty(roleIds) &&
                (CollectionUtils.isEmpty(resTypeEnums) || CollectionUtils.isEmpty(resIds))){
            throw new AppException("不合法的入参，删除权限失败！");
        }
        AuthPermissionExample example = buildExampleByRRO(resTypeEnums, resIds, operaEnums, roleIds);
        mapper.deleteByExample(example);
    }

    @Override
    public List<AuthPermission> findByRRO(List<ResourceConst.ResourceTypeEnum> resTypeEnums, List<String> resIds,
            List<ResourceConst.ResourceOperaEnum> operaEnums, List<String> roleIds){
        if(CollectionUtils.isEmpty(roleIds) && (CollectionUtils.isEmpty(resTypeEnums) || CollectionUtils.isEmpty(
                resIds))){
            //throw new AppException("不合法的入参，查询权限失败！");
            log.warn("不合法的入参，查询权限失败！直接返回null");
            return new ArrayList<>();
        }
        AuthPermissionExample example = buildExampleByRRO(resTypeEnums, resIds, operaEnums, roleIds);
        return mapper.selectByExample(example);
    }

    private static AuthPermissionExample buildExampleByRRO(List<ResourceConst.ResourceTypeEnum> resTypeEnums,
            List<String> resIds, List<ResourceConst.ResourceOperaEnum> operaEnums, List<String> roleIds){
        AuthPermissionExample example = new AuthPermissionExample();
        AuthPermissionExample.Criteria criteria = example.createCriteria();
        if(CollectionUtils.isNotEmpty(roleIds)){
            criteria.andRoleIdIn(roleIds);
        }
        if(CollectionUtils.isNotEmpty(resTypeEnums)){
            List<String> resTypes = resTypeEnums.stream().filter(Objects::nonNull)
                    .map(ResourceConst.ResourceTypeEnum::getKey).collect(Collectors.toList());
            if(CollectionUtils.isNotEmpty(resTypes)){
                criteria.andResTypeIn(resTypes);
            }
        }
        if(CollectionUtils.isNotEmpty(resIds)){
            criteria.andResIdIn(resIds);
        }
        if(CollectionUtils.isNotEmpty(operaEnums)){
            List<String> operas = operaEnums.stream().filter(Objects::nonNull)
                    .map(ResourceConst.ResourceOperaEnum::getKey).collect(Collectors.toList());
            if(CollectionUtils.isNotEmpty(operas)){
                criteria.andOperaTypeIn(operas);
            }
        }
        return example;
    }

}
