package com.kingtom.kirin.app.auth.service.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.regex.Pattern;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.kingtom.kirin.app.auth.dao.AuthRoleMapper;
import com.kingtom.kirin.app.auth.model.AuthRole;
import com.kingtom.kirin.app.auth.model.AuthRoleExample;
import com.kingtom.kirin.app.auth.model.AuthRoleExt3;
import com.kingtom.kirin.app.auth.model.AuthRoleSource;
import com.kingtom.kirin.app.auth.service.IAuthRoleService;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.exception.AppException;
import com.kingtom.kirin.core.common.json.JObject;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.common.utils.IDUtils;
import com.kingtom.kirin.core.common.utils.JsonUtils;
import com.kingtom.kirin.core.resource.ResourceConst;
import com.kingtom.kirin.core.resource.ResourceObject;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 权限模块</p>
 * 角色 pojo service impl</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/8/24
 */
@Service
public class AuthRoleServiceImpl implements IAuthRoleService{

    /**
     * 英文字母开头，由英文字母数字、下划线组成
     */
    private static final Pattern pattern = Pattern.compile("^[a-zA-Z][0-9a-zA-Z_-]*$");

    @Autowired
    private AuthRoleMapper mapper;

    @Override
    public AuthRole createAuthRole(AuthRole role){
        if(role == null){
            throw new AppException("角色数据为空，新建角色失败！");
        }
        if(StringUtils.isBlank(role.getName())){
            throw new AppException("角色名称为空，新建角色失败！");
        }
        if(StringUtils.isBlank(role.getCode())){
            throw new AppException("角色标识符为空，新建角色失败！");
        }
        if(!pattern.matcher(role.getCode()).matches()){
            throw new AppException("角色标识符格式错误，新建角色失败！");
        }
        if(StringUtils.isBlank(role.getSysId())){
            throw new AppException("系统Id为空，新建角色失败！");
        }
        AuthRole roleByName = findAuthRoleByName(role.getName(), role.getSysId());
        if(roleByName != null){
            throw new AppException("角色名称重复，新建角色失败！");
        }
        AuthRole roleByCode = findAuthRoleByCode(role.getName(), role.getSysId());
        if(roleByCode != null){
            throw new AppException("角色标识符重复，新建角色失败！");
        }
        fillBeforeSave(role, true);
        if(role.getState() != null && CommonConst.StateEnum.findEnumByKey(role.getState()) == null){
            throw new AppException("角色状态错误，新建角色失败！");
        }
        boolean f = mapper.insertSelective(role) == 1;
        if(!f){
            throw new AppException("落库出错，新建角色失败！");
        }
        return mapper.selectByPrimaryKey(role.getId());
    }

    private static void fillBeforeSave(AuthRole role, boolean isCreate){

        role.setGmtModified(new Date());
        if(isCreate){
            if(CommonConst.BooleanEnum.findEnumByKey(role.getDisplay()) == null){
                role.setDisplay(CommonConst.BooleanEnum.False.getKey());
            }
            if(role.getExt3() == null){
                role.setExt3(new JObject<>(new AuthRoleExt3()));
            }
            if(role.getSource() == null){
                role.setSource(new JObject<>(new AuthRoleSource(new ArrayList<>())));
            }
            role.setId(IDUtils.newUUID());
            role.setGmtCreate(role.getGmtModified());
            if(role.getState() == null){
                role.setState(CommonConst.StateEnum.Enable.getKey());
            }
        }
    }

    @Override
    public AuthRole updateAuthRoleById(AuthRole role){
        if(role == null || StringUtils.isEmpty(role.getId())){
            throw new AppException("角色信息错误，更新角色失败！");
        }
        if(role.getName() != null && StringUtils.isBlank(role.getName())){
            throw new AppException("角色名称为空，更新角色失败！");
        }
        if(role.getCode() != null){
            if(StringUtils.isBlank(role.getCode())){
                throw new AppException("角色标识符为空，更新角色失败！");
            }
            if(!pattern.matcher(role.getCode()).matches()){
                throw new AppException("角色标识符格式错误，更新角色失败！");
            }
        }
        CommonConst.StateEnum stateEnum = CommonConst.StateEnum.findEnumByKey(role.getState());
        if(role.getState() != null && stateEnum == null){
            throw new AppException("角色状态错误，更新角色失败！");
        }
        fillBeforeSave(role, false);
        AuthRole updateRole = new AuthRole();
        updateRole.setId(role.getId());
        updateRole.setName(role.getName());
        updateRole.setCode(role.getCode());
        updateRole.setSource(role.getSource());
        updateRole.setExt1(role.getExt1());
        updateRole.setExt2(role.getExt2());
        updateRole.setExt3(role.getExt3());
        updateRole.setGmtModified(role.getGmtModified());
        boolean f = mapper.updateByPrimaryKeySelective(updateRole) == 1;
        if(!f){
            throw new AppException("落库出错，更新角色失败！");
        }
        return findAuthRoleById(role.getId());
    }

    @Override
    public AuthRole updateAuthRoleAddBindResource(String roleId, ResourceObject<String> resources){
        boolean f = mapper.updateAuthRoleAddBindResource(roleId, JsonUtils.toJson(resources), new Date()) == 1;
        if(!f){
            throw new AppException("落库出错，更新角色追加绑定资源失败！");
        }
        return findAuthRoleById(roleId);
    }

    @Override
    public AuthRole updateAuthRoleAddBindResources(String roleId, List<ResourceObject<String>> resources){
        boolean f = mapper.updateAuthRoleAddBindResource(roleId, JsonUtils.toJson(resources), new Date()) == 1;
        if(!f){
            throw new AppException("落库出错，更新角色追加绑定资源失败！");
        }
        return findAuthRoleById(roleId);
    }

    @Override
    public AuthRole updateAuthRoleRemoveBindResource(String roleId, ResourceObject<String> resources){
        boolean f =
                mapper.updateAuthRoleRemoveBindResources(roleId, Collections.singletonList(resources), new Date()) == 1;
        if(!f){
            throw new AppException("落库出错，更新角色追加绑定资源失败！");
        }
        return findAuthRoleById(roleId);
    }

    @Override
    public AuthRole updateAuthRoleRemoveBindResources(String roleId, List<ResourceObject<String>> resources){
        boolean f = mapper.updateAuthRoleRemoveBindResources(roleId, resources, new Date()) == 1;
        if(!f){
            throw new AppException("落库出错，更新角色追加绑定资源失败！");
        }
        return findAuthRoleById(roleId);
    }

    @Override
    public AuthRole updateAuthRoleUpdateBindResources(String roleId, List<ResourceObject<String>> resources){
        boolean f = mapper.updateAuthRoleUpdateBindResources(roleId, JsonUtils.toJson(resources), new Date()) == 1;
        if(!f){
            throw new AppException("落库出错，更新角色更新绑定资源失败！");
        }
        return findAuthRoleById(roleId);
    }

    @Override
    public void updateAuthRoleAddBindResourcesByCodes(List<String> roleCode, List<ResourceObject<String>> resources){
        boolean f = mapper.updateAuthRoleAddBindResourcesByCodes(roleCode, JsonUtils.toJson(resources), new Date()) ==
                roleCode.size();
        if(!f){
            throw new AppException("落库出错，更新角色更新绑定资源失败！");
        }
    }

    @Override
    public void updateAuthRoleRemoveBindResourcesByCodes(List<String> roleCode, List<ResourceObject<String>> resources){
        //FIXME sql 有严重bug
//        boolean f = mapper.updateAuthRoleRemoveBindResourcesByCodes(roleCode, resources, new Date()) == roleCode.size();
//        if(!f){
//            throw new AppException("落库出错，更新角色更新绑定资源失败！");
//        }
    }

    @Override
    public void deleteAuthRoleById(String roleId){
        if(StringUtils.isBlank(roleId)){
            return;
        }
        AuthRole role = new AuthRole();
        role.setId(roleId);
        role.setState(CommonConst.StateEnum.Deleted.getKey());
        role.setGmtModified(new Date());
        boolean f = mapper.updateByPrimaryKeySelective(role) == 1;
        if(!f){
            throw new AppException("落库出错，删除角色失败！");
        }
    }

    @Override
    public void deleteRoles(List<String> ids){
        if(CollectionUtils.isEmpty(ids)){
            return;
        }
        AuthRoleExample example = new AuthRoleExample();
        example.createCriteria().andIdIn(ids);
        AuthRole role = new AuthRole();
        role.setState(CommonConst.StateEnum.Deleted.getKey());
        role.setGmtModified(new Date());
        boolean f = mapper.updateByExampleSelective(role, example) == ids.size();
        if(!f){
            throw new AppException("落库出错，删除角色失败！");
        }
    }

    @Override
    public AuthRole findAuthRoleById(String roleId){
        if(StringUtils.isBlank(roleId)){
            return null;
        }
        AuthRole role = mapper.selectByPrimaryKey(roleId);
        return role != null && !CommonConst.StateEnum.Deleted.getKey().equals(role.getState()) ? role : null;
    }

    @Override
    public AuthRole findAuthRoleByCode(String code, String sysId){
        if(StringUtils.isAnyBlank(code, sysId)){
            return null;
        }
        AuthRoleExample example = new AuthRoleExample();
        AuthRoleExample.Criteria criteria = example.createCriteria();
        criteria.andCodeEqualTo(code);
        criteria.andSysIdEqualTo(sysId);
        excludeDeleteData(example);
        List<AuthRole> authRoles = mapper.selectByExample(example);
        return CollectionUtils.isEmpty(authRoles) ? null : authRoles.get(CommonConst.INT_0);
    }

    @Override
    public AuthRole findAuthRoleByName(String name, String sysId){
        if(StringUtils.isAnyBlank(name, sysId)){
            return null;
        }
        AuthRoleExample example = new AuthRoleExample();
        AuthRoleExample.Criteria criteria = example.createCriteria();
        criteria.andNameEqualTo(name);
        criteria.andSysIdEqualTo(sysId);
        excludeDeleteData(example);
        List<AuthRole> authRoles = mapper.selectByExample(example);
        return CollectionUtils.isEmpty(authRoles) ? null : authRoles.get(CommonConst.INT_0);

    }

    @Override
    public PageInfo<AuthRole> findRolePage(AuthRoleExample example, int pageNum, int pageSize){
        if(example == null){
            return null;
        }
        PageHelper.startPage(pageNum, pageSize, "gmt_create desc");
        excludeDeleteData(example);
        List<AuthRole> list = mapper.selectByExample(example);
        return new PageInfo<>(list);
    }

    @Override
    public List<AuthRole> findAll(String sysId){
        AuthRoleExample example = new AuthRoleExample();
        AuthRoleExample.Criteria criteria = example.createCriteria();
        if(StringUtils.isNotEmpty(sysId)){
            criteria.andSysIdEqualTo(sysId);
        }
        excludeDeleteData(example);
        return mapper.selectByExample(example);
    }

    @Override
    public List<AuthRole> findBySource(ResourceConst.ResourceTypeEnum resourceTypeEnum, String resId, String sysId){
        AuthRoleExample example = new AuthRoleExample();
        AuthRoleExample.Criteria criteria = example.createCriteria();
        if(StringUtils.isNotEmpty(sysId)){
            criteria.andSysIdEqualTo(sysId);
        }
        criteria.addCriterion(
                "source::jsonb @> " + "'{\"e" + "\":{\"resources\":[{\"resType\":\"" + resourceTypeEnum.getKey() +
                        "\",\"resId\":\"" + resId + "\"}]}}'");
        excludeDeleteData(example);
        return mapper.selectByExample(example);
    }

    /**
     * 排除掉被删除的数据
     *
     * @param example
     */
    private static void excludeDeleteData(AuthRoleExample example){
        example.getOredCriteria().forEach(e -> e.andStateNotEqualTo(CommonConst.StateEnum.Deleted.getKey()));
    }

}
