package com.kingtom.kirin.app.auth.service.impl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import com.github.pagehelper.PageInfo;
import com.kingtom.kirin.app.auth.AuthConst;
import com.kingtom.kirin.app.auth.model.*;
import com.kingtom.kirin.app.auth.service.IAuthMenuService;
import com.kingtom.kirin.app.auth.service.IAuthPermissionService;
import com.kingtom.kirin.app.auth.service.IAuthRoleService;
import com.kingtom.kirin.app.auth.service.IAuthService;
import com.kingtom.kirin.app.auth.utils.AuthUtils;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.exception.AppException;
import com.kingtom.kirin.core.common.json.JObject;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.resource.ResourceConst;
import com.kingtom.kirin.core.resource.ResourceObject;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 权限模块</p>
 * 身份授权认证 接口 实现类</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023 /8/29
 */
@Service
public class AuthServiceImpl implements IAuthService{

    @Autowired
    private IAuthRoleService authRoleService;

    @Autowired
    private IAuthMenuService authMenuService;

    @Autowired
    private IAuthPermissionService authPermissionService;

    @Override
    public AuthRole addRole(AuthRole role, List<String> menuIds){
        //创建角色
        if(role.getSource() == null){
            role.setSource(new JObject<>(new AuthRoleSource(new ArrayList<>())));
        }
        if(role.getSource().getE() == null){
            role.getSource().setE(new AuthRoleSource(new ArrayList<>()));
        }
        if(role.getSource().getE().getResources() == null){
            role.getSource().getE().setResources(new ArrayList<>());
        }
        List<ResourceObject<String>> resources = role.getSource().getE().getResources();
        //todo 调用资源模块检查资源是否存在
        AuthRole r = authRoleService.createAuthRole(role);
        //绑定菜单权限
        if(CollectionUtils.isEmpty(menuIds)){
            return r;
        }
        List<String> dbMenuIds = new ArrayList<>();
        updatePermissionForBindMenu(r.getId(), menuIds, dbMenuIds);
        return r;
    }

    /**
     * 角色绑定菜单
     *
     * @param roleId    角色Id
     * @param menuIds   前端提交的需要绑定的菜单，
     * @param dbMenuIds 数据库当前角色已经绑定的菜单
     */
    @Override
    public void updatePermissionForBindMenu(String roleId, List<String> menuIds, List<String> dbMenuIds){
        List<List<String>> list = CollectionUtils.analyzeCollection(dbMenuIds, menuIds, String::equals);
        List<String> same = list.get(0);
        //删除权限
        List<String> remove = list.get(1);
        if(CollectionUtils.isNotEmpty(remove)){
            List<ResourceConst.ResourceTypeEnum> resourceTypeEnums = List.of(ResourceConst.ResourceTypeEnum.AuthMenu);
            List<ResourceConst.ResourceOperaEnum> operaEnums = List.of(ResourceConst.ResourceOperaEnum.Execute);
            List<String> roleIds = Collections.singletonList(roleId);
            authPermissionService.deleteByRRO(resourceTypeEnums, remove, operaEnums, roleIds);
        }
        //新增权限
        List<String> add = list.get(2);
        if(CollectionUtils.isNotEmpty(add)){
            List<AuthMenu> addMenu = authMenuService.findAuthMenuByIds(add);
            List<AuthPermission> permissions =
                    addMenu.stream().map(menu -> createAuthPermissionBean(roleId, menu)).collect(Collectors.toList());
            authPermissionService.createBatch(permissions);
        }
    }

    /**
     * 角色绑定菜单，创建一个权限实例
     *
     * @param roleId
     * @param menu
     * @return
     */
    private AuthPermission createAuthPermissionBean(String roleId, AuthMenu menu){
        AuthPermission permission = new AuthPermission();
        permission.setResType(ResourceConst.ResourceTypeEnum.AuthMenu.getKey());
        permission.setResId(menu.getId());
        permission.setOperaType(ResourceConst.ResourceOperaEnum.Execute.getKey());
        permission.setRoleId(roleId);
        permission.setSysId(menu.getSysId());
        permission.setCode(AuthUtils.buildPermissionCode(permission, menu.getCode()));
        return permission;
    }

    @Override
    public void deleteRoleById(String id){
        authRoleService.deleteAuthRoleById(id);
        authPermissionService.deleteByRRO(null, null, null, Collections.singletonList(id));
    }

    @Override
    public void deleteRoleByIds(List<String> ids){
        authRoleService.deleteRoles(ids);
        authPermissionService.deleteByRRO(null, null, null, ids);
    }

    @Override
    public AuthRole updateRole(AuthRole role, List<String> menuIds){
        if(role.getSource() != null){
            //todo 调用资源模块检查资源是否存在
        }
        AuthRole r = authRoleService.updateAuthRoleById(role);
        //更新绑定的菜单
        List<ResourceConst.ResourceTypeEnum> resTypeEnums = List.of(ResourceConst.ResourceTypeEnum.AuthMenu);
        List<ResourceConst.ResourceOperaEnum> operaEnums = List.of(ResourceConst.ResourceOperaEnum.Execute);
        List<AuthPermission> permissions =
                authPermissionService.findByRRO(resTypeEnums, null, operaEnums, Collections.singletonList(r.getId()));
        List<String> dbMenuIds =
                permissions.stream().map(AuthPermission::getResId).distinct().collect(Collectors.toList());
        updatePermissionForBindMenu(r.getId(), menuIds, dbMenuIds);
        return r;
    }

    @Override
    public AuthRole updateAuthRoleAddBindResource(String roleId, ResourceObject<String> resourceObject){
        return authRoleService.updateAuthRoleAddBindResource(roleId, resourceObject);
    }

    @Override
    public AuthRole findRoleById(String id){
        return authRoleService.findAuthRoleById(id);
    }

    @Override
    public AuthRole findSuperAdminRole(String sysId){
        return authRoleService.findAuthRoleByCode(AuthConst.SUPER_ADMIN_ROLE_US_NAME, sysId);
    }

    @Override
    public AuthRole findUserRole(String sysId){
        return authRoleService.findAuthRoleByCode(AuthConst.USER_ROLE_US_NAME, sysId);
    }

    @Override
    public List<AuthRole> findAllRole(String sysId){
        return authRoleService.findAll(sysId);
    }

    @Override
    public PageInfo<AuthRole> findRolePage(String search, int pageNum, int pageSize){
        AuthRoleExample example = new AuthRoleExample();
        AuthRoleExample.Criteria criteria = example.createCriteria();
        if(StringUtils.isNotBlank(search)){
            criteria.andNameLike("%" + search + "%");
            AuthRoleExample.Criteria criteria1 = example.or();
            criteria1.andCodeLike("%" + search + "%");
        }
        //排除掉超级管理员
        criteria.andDisplayEqualTo(CommonConst.BooleanEnum.True.getKey());
        criteria.andCodeNotEqualTo(AuthConst.SUPER_ADMIN_ROLE_US_NAME);
        return authRoleService.findRolePage(example, pageNum, pageSize);
    }

    @Override
    public List<AuthRole> findRoleBySystemUser(String userId, String sysId){
        return authRoleService.findBySource(ResourceConst.ResourceTypeEnum.SystemUser, userId, sysId);
    }

    @Override
    public AuthMenu addMenu(AuthMenu menu){
        AuthMenu res = authMenuService.createAuthMenu(menu);
        createPermissionForAdminRoleBindMenu(Arrays.asList(res));
        return res;
    }

    @Override
    public List<AuthMenu> addMenus(List<AuthMenu> menus){
        List<AuthMenu> list = authMenuService.createBatchAuthMenu(menus);
        createPermissionForAdminRoleBindMenu(menus);
        return list;
    }

    /**
     * 给超级管理员绑定菜单权限
     *
     * @param list
     */
    private void createPermissionForAdminRoleBindMenu(List<AuthMenu> list){
        if(CollectionUtils.isEmpty(list)){
            return;
        }
        AuthRole role = findSuperAdminRole(list.get(0).getSysId());
        if(role == null){
            return;
        }
        List<AuthPermission> permissions =
                list.stream().map(e -> createAuthPermissionBean(role.getId(), e)).collect(Collectors.toList());
        authPermissionService.createBatch(permissions);
    }

    @Override
    public void deleteMenuById(String id, boolean deleteChild){
        //找到菜单下的所有子菜单，
        List<String> ids = new ArrayList<>();
        ids.add(id);
        if(deleteChild){
            List<AuthMenu> authMenus = authMenuService.findChildren(id);
            ids.addAll(authMenus.stream().map(AuthMenu::getId).collect(Collectors.toList()));
        }
        authMenuService.deleteAuthMenuByIds(ids);
        //删除相关权限
        deleteAuthPermissionByMenuIds(ids);
    }

    @Override
    public void deleteMenuByIds(List<String> ids){

        authMenuService.deleteAuthMenuByIds(ids);
        //删除相关权限
        deleteAuthPermissionByMenuIds(ids);
    }

    /**
     * 根据菜单id，删除对应的权限
     *
     * @param ids
     */
    private void deleteAuthPermissionByMenuIds(List<String> ids){
        List<ResourceConst.ResourceTypeEnum> resTypeEnums = List.of(ResourceConst.ResourceTypeEnum.AuthMenu);
        List<ResourceConst.ResourceOperaEnum> operaEnums = List.of(ResourceConst.ResourceOperaEnum.Execute);
        authPermissionService.deleteByRRO(resTypeEnums, ids, operaEnums, null);
    }

    @Override
    public AuthMenu updateMenu(AuthMenu menu){

        AuthMenu dbMenu = authMenuService.findAuthMenuById(menu.getId());
        if(dbMenu == null){
            throw new AppException("菜单不存在，更新菜单失败！");
        }
        AuthMenu update = authMenuService.updateAuthMenuById(menu);
        //更新权限码
        if(StringUtils.isNotBlank(update.getCode()) && !update.getCode().equals(dbMenu.getCode())){
            List<ResourceConst.ResourceTypeEnum> resTypeEnums = List.of(ResourceConst.ResourceTypeEnum.AuthMenu);
            List<ResourceConst.ResourceOperaEnum> operaEnums = List.of(ResourceConst.ResourceOperaEnum.Execute);
            List<AuthPermission> permissions =
                    authPermissionService.findByRRO(resTypeEnums, Arrays.asList(update.getId()), operaEnums, null);
            if(CollectionUtils.isNotEmpty(permissions)){
                for(AuthPermission permission : permissions){
                    AuthPermission permission1 = new AuthPermission();
                    permission1.setId(permission.getId());
                    permission1.setResType(ResourceConst.ResourceTypeEnum.AuthMenu.getKey());
                    permission1.setOperaType(ResourceConst.ResourceOperaEnum.Execute.getKey());
                    permission1.setCode(AuthUtils.buildPermissionCode(permission1, update.getCode()));
                    authPermissionService.update(permission1);
                }
            }
        }
        return update;
    }

    @Override
    public AuthMenu findMenuById(String id){
        return authMenuService.findAuthMenuById(id);
    }

    @Override
    public List<AuthMenu> findMenuListBySystem(CommonConst.BooleanEnum isPreset, String sysId){
        return authMenuService.findMenuBySysId(isPreset, sysId);
    }

    @Override
    public List<AuthMenu> findMenuListBySystemRuntime(String userId, String sysId){
        List<ResourceConst.ResourceTypeEnum> resTypeEnums = List.of(ResourceConst.ResourceTypeEnum.AuthMenu);
        List<ResourceConst.ResourceOperaEnum> operaEnums = List.of(ResourceConst.ResourceOperaEnum.Execute);
        List<AuthPermission> list = findAuthPermissions(resTypeEnums, operaEnums, userId, sysId);
        List<String> menuIds = list.stream().map(AuthPermission::getResId).collect(Collectors.toList());
        List<AuthMenu> menus = authMenuService.findAuthMenuByIds(menuIds);
        //把父菜单加入进入，不管有没有授权。
        List<String> parentIds = menus.stream().flatMap(e -> {
            List<String> array = new ArrayList<>();
            array.add(e.getId());
            array.addAll(e.getRoute().getE().getPath());
            return array.stream();
        }).distinct().filter(e -> e != null && !menuIds.contains(e)).collect(Collectors.toList());
        if(CollectionUtils.isNotEmpty(parentIds)){
            List<AuthMenu> parentMenus = authMenuService.findAuthMenuByIds(parentIds);
            if(CollectionUtils.isNotEmpty(parentMenus)){
                menus.addAll(parentMenus);
            }
        }
        return menus;
    }

    @Override
    public List<AuthMenu> findMenuListByIds(List<String> ids){
        return authMenuService.findAuthMenuByIds(ids);
    }

    @Override
    public List<String> findBindMenuByRoleId(String id){
        List<ResourceConst.ResourceTypeEnum> resTypeEnums = List.of(ResourceConst.ResourceTypeEnum.AuthMenu);
        List<ResourceConst.ResourceOperaEnum> operaEnums = List.of(ResourceConst.ResourceOperaEnum.Execute);
        List<AuthPermission> permissions =
                authPermissionService.findByRRO(resTypeEnums, null, operaEnums, Collections.singletonList(id));
        return permissions.stream().map(AuthPermission::getResId).distinct().collect(Collectors.toList());
    }

    @Override
    public List<AuthPermission> findAllAuthPermissions(String userId, String sysId){
        return findAuthPermissions(null, null, userId, sysId);
    }

    @Override
    public List<AuthPermission> findAuthPermissions(List<ResourceConst.ResourceTypeEnum> resourceTypeEnums,
            List<ResourceConst.ResourceOperaEnum> operaEnums, String userId, String sysId){
        List<AuthRole> role = findRoleBySystemUser(userId, sysId);
        List<String> roleIds = role.stream().map(AuthRole::getId).collect(Collectors.toList());
        List<AuthPermission> permissions =
                authPermissionService.findByRRO(resourceTypeEnums, null, operaEnums, roleIds);
        return permissions;
    }

}
