package com.kingtom.kirin.app.ddic.service;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.kingtom.kirin.app.ddic.dao.DdicInfoMapper;
import com.kingtom.kirin.app.ddic.dao.DdicItemMapper;
import com.kingtom.kirin.app.ddic.mapper.DdicMapper;
import com.kingtom.kirin.app.ddic.model.*;
import com.kingtom.kirin.app.ddic.utils.DdicUtils;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.exception.AppException;
import com.kingtom.kirin.core.common.utils.CaseUtils;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.common.utils.ObjectUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.List;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/8/26 14:15
 */
@Component
@Slf4j
public class DdicServiceImpl implements IDdicService{

    @Autowired
    private DdicInfoMapper ddicInfoMapper;
    @Autowired
    private DdicItemMapper ddicItemMapper;

    @Override
    public DdicInfo createDdicInfo(DdicInfo info) {
        CaseUtils.build(info,
                CaseUtils.caseBeanAndThrow(ObjectUtil::isNull,"字典不能为空"),
                CaseUtils.caseAttrBlankAndThrow(DdicInfo::getName,"字典名不能为空"),
                CaseUtils.caseAttrBlankAndThrow(DdicInfo::getCode,"字典编码不能为空"),
                CaseUtils.caseRelCheckAndThrow(b->findDdicInfoByName(b.getName()), ObjectUtil::isNotNull,"字典名不能重复"),
                CaseUtils.caseRelCheckAndThrow(b->findDdicInfoByCode(b.getCode()), ObjectUtil::isNotNull,"字典编码不能重复"))
                .doCase();
        DdicUtils.fillBeforeCreate(info);
        ddicInfoMapper.insert(info);
        log.info("字典["+info.getName()+"]创建完成");
        return info;
    }

    @Override
    public DdicInfo updateDdicInfo(DdicInfo info) {
        CaseUtils.Context<DdicInfo> context = CaseUtils.build(info,
                CaseUtils.caseBeanAndThrow(ObjectUtil::isNull,"字典不能为空"),
                CaseUtils.caseAttrBlankAndThrow(DdicInfo::getId,"字典不能为空"),
                CaseUtils.caseRelCheckAndThrow(b->findDdicInfoById(b.getId()), ObjectUtil::isNull,"字典不存在"),
                CaseUtils.caseAttrBlankAndThrow(DdicInfo::getName,"字典名不能为空"),
                CaseUtils.caseAttrBlankAndThrow(DdicInfo::getCode,"字典编码不能为空"),
                CaseUtils.caseRelCompareAndThrow(b->findDdicInfoByName(b.getName()),DdicUtils::isNotSame,"字典名不能重复"),
                CaseUtils.caseRelCompareAndThrow(b->findDdicInfoByCode(b.getCode()),DdicUtils::isNotSame,"字典编码不能重复"))
                .doCase();
        DdicInfo dbInfo = context.getCache(CommonConst.INT_2);
        DdicInfo copyDbBean = DdicMapper.INSTANCT.copyDepth(dbInfo);
        ObjectUtils.copyPropertiesByCover(info,copyDbBean,"name","simpleName","code","ext3","des");
        DdicUtils.fillBeforeUpdate(copyDbBean,dbInfo);
        ddicInfoMapper.updateByPrimaryKey(copyDbBean);
        log.info("字典["+info.getName()+"]更新完成");
        return copyDbBean;
    }

    @Override
    public DdicInfo deleteDdicInfo(String id) {
        DdicInfo dbInfo = findDdicInfoById(id);
        if(dbInfo != null) {
            ddicInfoMapper.deleteByPrimaryKey(id);
            DdicItemExample e = new DdicItemExample();
            e.createCriteria().andDdicIdEqualTo(id);
            ddicItemMapper.deleteByExample(e);
        } else {
            log.warn("字典["+id+"]已经不存在，无需删除");
        }
        return dbInfo;
    }

    @Override
    public PageInfo<DdicInfo> findDdicInfos(String search, Integer pageNum, Integer pageSize) {
        DdicInfoExample e = new DdicInfoExample();
        if(StrUtil.isNotBlank(search)) {
            e.or().andNameLike("%"+search+"%");
            e.or().andSimpleNameLike("%"+search+"%");
            e.or().andCodeLike("%"+search+"%");
        }
        e.setOrderByClause("gmt_create desc");
        PageHelper.startPage(pageNum, pageSize);
        List<DdicInfo> infos = ddicInfoMapper.selectByExample(e);
        return new PageInfo<>(infos);
    }

    @Override
    public DdicInfo findDdicInfoById(String ddicId) {
        return StrUtil.isBlank(ddicId)?null:ddicInfoMapper.selectByPrimaryKey(ddicId);
    }

    @Override
    public DdicItem createDdicItem(DdicItem info) {
        CaseUtils.Context<DdicItem> context = CaseUtils.build(info,
                CaseUtils.caseBeanAndThrow(ObjectUtil::isNull,"字典项不能为空"),
                CaseUtils.caseRelAndThrow(b->findDdicInfoById(b.getDdicId()),ObjectUtil::isNull,"字典归属不存在", DdicItem::getDdicId),
                CaseUtils.caseAttrBlankAndThrow(DdicItem::getName,"字典项名不能为空"),
                CaseUtils.caseAttrBlankAndThrow(DdicItem::getKey,"字典项KEY不能为空"),
                CaseUtils.caseBeanAndThrow(b->b.getSort() != null && b.getSort().compareTo(BigDecimal.ONE) < CommonConst.INT_0,"字典项排序号必须大于0"))
                .doCase();
        DdicInfo dbDdicInfo = context.getCache(CommonConst.INT_1);
        if(findDdicItemByKey(dbDdicInfo.getCode(),info.getKey()) != null) {
            throw new AppException("字典项KEY不能重复").setM(info.getKey());
        }
        boolean isTop = DdicUtils.isDdicItemTop(info);
        DdicItem parent = isTop?null:findDdicItemByDdicIdAndKey(dbDdicInfo.getId(),info.getParentKey());
        if( !isTop && parent == null) {
            throw new AppException("字典项父级不存在").setM(info.getParentKey());
        }
        DdicUtils.fillBeforeCreate(info, parent);
        DdicItem preBean = findDdicItemByDdicIdAndLevelAndParentOfPre(info.getDdicId(), info.getParentKey(), info.getLevel());
        DdicUtils.fillBeforeCreateOfSort(info, parent, preBean);
        ddicItemMapper.insert(info);
        return info;
    }

    @Override
    public DdicItem updateDdicItem(DdicItem info) {
        CaseUtils.Context<DdicItem> context = CaseUtils.build(info,
                        CaseUtils.caseBeanAndThrow(ObjectUtil::isNull,"字典项不能为空"),
                        CaseUtils.caseRelAndThrow(b->findDdicItemById(b.getId()),ObjectUtil::isNull,"字典项不存在"),
                        CaseUtils.caseRelAndThrow(b->findDdicInfoById(b.getDdicId()),ObjectUtil::isNull,"字典归属不存在", DdicItem::getDdicId),
                        CaseUtils.caseAttrBlankAndThrow(DdicItem::getName,"字典项名不能为空"),
                        CaseUtils.caseAttrBlankAndThrow(DdicItem::getKey,"字典项KEY不能为空"),
                        CaseUtils.caseBeanAndThrow(b->b.getSort() != null && b.getSort().compareTo(BigDecimal.ONE) < CommonConst.INT_0,"字典项排序号必须大于0"))
                .doCase();
        DdicItem dbInfo = context.getCache(CommonConst.INT_1);
        DdicInfo dbDdicInfo = context.getCache(CommonConst.INT_2);
        DdicItem reInfo = findDdicItemByKey(dbDdicInfo.getCode(),info.getKey());
        if(reInfo != null && ObjectUtil.notEqual(reInfo.getId(),info.getId())) {
            throw new AppException("字典项KEY不能重复").setM(info.getKey());
        }
        boolean isTop = DdicUtils.isDdicItemTop(info);
        DdicItem parent = isTop?null:findDdicItemByDdicIdAndKey(dbDdicInfo.getId(),info.getParentKey());
        if( !isTop && parent == null) {
            throw new AppException("字典项父级不存在").setM(info.getParentKey());
        }
        DdicItem copyDbBean = DdicMapper.INSTANCT.copyDepth(dbInfo);
        ObjectUtils.copyPropertiesByCover(info,copyDbBean,"name","sort","ddicId","parentKey","des","key","ext1","ext2","ext3");
        DdicUtils.fillBeforeUpdate(copyDbBean,parent,dbInfo);
        DdicItem preBean = findDdicItemByDdicIdAndLevelAndParentOfPre(info.getDdicId(), info.getParentKey(), info.getLevel());
        DdicUtils.fillBeforeUpdateOfSort(copyDbBean,parent,preBean,dbInfo);
        ddicItemMapper.updateByPrimaryKey(copyDbBean);
        return copyDbBean;
    }

    @Override
    public DdicItem deleteDdicItem(String id) {
        DdicItem dbInfo = findDdicItemById(id);
        if(dbInfo != null) {
            ddicItemMapper.deleteByPrimaryKey(id);
        } else {
            log.warn("字典项["+id+"]已经不存在，无需删除");
        }
        return dbInfo;
    }

    @Override
    public PageInfo<DdicItem> findDdicItems(String search, String ddicId, Integer pageNum, Integer pageSize) {
        DdicItemExample e = new DdicItemExample();
        ddicId = StrUtil.isBlank(ddicId)?CommonConst.UNKOWN:ddicId;
        if(StrUtil.isNotBlank(search)) {
            e.or().andNameLike("%"+search+"%").andDdicIdEqualTo(ddicId);
            e.or().andSimpleNameLike("%"+search+"%").andDdicIdEqualTo(ddicId);
            e.or().andKeyLike("%"+search+"%").andDdicIdEqualTo(ddicId);
        } else {
            e.createCriteria().andDdicIdEqualTo(ddicId);
        }
        e.setOrderByClause("sort desc");
        PageHelper.startPage(pageNum, pageSize);
        List<DdicItem> infos = ddicItemMapper.selectByExample(e);
        return new PageInfo<>(infos);
    }

    @Override
    public DdicItem findDdicItemById(String id) {
        return StrUtil.isBlank(id)?null:ddicItemMapper.selectByPrimaryKey(id);
    }

    @Override
    public DdicItem findDdicItemByKey(String ddicCode, String key) {
        if(StrUtil.isAllNotBlank(ddicCode,key)) {
            DdicInfo info = findDdicInfoByCode(ddicCode);
            if(info != null) {
                DdicItemExample e = new DdicItemExample();
                e.createCriteria().andDdicIdEqualTo(info.getId()).andKeyEqualTo(key);
                return CollectionUtils.findFirst(ddicItemMapper.selectByExample(e));
            } else {
                return null;
            }
        } else {
            return null;
        }
    }

    @Override
    public DdicInfo findDdicInfoByCode(String ddicCode) {
        DdicInfoExample e = new DdicInfoExample();
        e.createCriteria().andCodeEqualTo(ddicCode);
        return CollectionUtils.findFirst(ddicInfoMapper.selectByExample(e));
    }

    @Override
    public DdicInfo findDdicInfoByName(String ddicName) {
        DdicInfoExample e = new DdicInfoExample();
        e.createCriteria().andNameEqualTo(ddicName);
        return CollectionUtils.findFirst(ddicInfoMapper.selectByExample(e));
    }

    @Override
    public DdicInfoDO findDdicInfoDOByCode(String ddicCode) {
        DdicInfo info = findDdicInfoByCode(ddicCode);
        if(info != null) {
            List<DdicInfoDO> dos = fillDos(ListUtil.toList(info));
            return CollectionUtils.findFirst(dos);
        }
        return null;
    }

    public DdicItem findDdicItemByDdicIdAndKey(String ddicId, String key) {
        if(StrUtil.isAllNotBlank(ddicId,key)) {
            DdicItemExample e = new DdicItemExample();
            e.createCriteria().andDdicIdEqualTo(ddicId).andKeyEqualTo(key);
            return CollectionUtils.findFirst(ddicItemMapper.selectByExample(e));
        } else {
            return null;
        }
    }

    public DdicItem findDdicItemByDdicIdAndLevelAndParentOfPre(String ddicId, String parentKey, Integer level) {
        if(ObjectUtil.isAllNotEmpty(ddicId,level)) {
            DdicItemExample e = new DdicItemExample();
            if(parentKey == null) {
                e.createCriteria().andDdicIdEqualTo(ddicId).andLevelEqualTo(level);
            } else {
                e.createCriteria().andDdicIdEqualTo(ddicId).andLevelEqualTo(level).andParentKeyEqualTo(parentKey);
            }
            e.setOrderByClause("gmt_create desc");
            PageHelper.startPage(CommonConst.INT_1, CommonConst.INT_1);
            List<DdicItem> infos = ddicItemMapper.selectByExample(e);
            return CollectionUtils.findFirst(infos);
        } else {
            return null;
        }
    }

    protected List<DdicItem> findDdicItemsByDdicId(String ddicId) {
        DdicInfo info = findDdicInfoById(ddicId);
        if(info != null) {
            DdicItemExample e = new DdicItemExample();
            e.createCriteria().andDdicIdEqualTo(ddicId);
            e.setOrderByClause("sort desc");
            List<DdicItem> beans = ddicItemMapper.selectByExample(e);
            return beans;
        } else {
            return ListUtil.toList();
        }
    }

    protected List<DdicInfoDO> fillDos(List<DdicInfo> beans) {
        List<DdicInfoDO> r = ListUtil.toList();
        if(CollectionUtil.isNotEmpty(beans)) {
            for(DdicInfo bean:beans) {
                DdicInfoDO doobj = new DdicInfoDO(bean);
                r.add(doobj);
                List<DdicItem> items = findDdicItemsByDdicId(bean.getId());
                doobj.setDdicItems(items);
            }
        }
        return r;
    }
}
