package com.kingtom.kirin.app.form.engine.generator;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.map.MapUtil;
import com.kingtom.kirin.app.form.engine.generator.impl.FormJavaGenerator;
import com.kingtom.kirin.app.form.engine.generator.tools.MemoryFileManager;
import com.kingtom.kirin.app.form.engine.generator.tools.MemoryJavaFileObject;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.exception.AppException;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import freemarker.cache.ClassTemplateLoader;
import freemarker.template.Configuration;
import freemarker.template.Template;
import freemarker.template.TemplateExceptionHandler;
import lombok.extern.slf4j.Slf4j;

import javax.sql.DataSource;
import javax.tools.*;
import java.io.StringWriter;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/10/14 12:18
 */
@Slf4j
public class FormGeneratorContext {

    private Configuration configuration;

    private JavaCompiler javaCompiler;

    private Map<String,Object> cache = MapUtil.newHashMap();

    public FormGeneratorContext() {
        this(null,null);
    }

    public FormGeneratorContext(Configuration configuration,JavaCompiler javaCompiler) {
        this.configuration = configuration == null?initFreeMarker():configuration;
        this.javaCompiler = javaCompiler == null?initJavaCompiler():javaCompiler;
    }

    protected Configuration initFreeMarker() {
        Configuration c = new Configuration(Configuration.getVersion());
        c.setTemplateLoader(new ClassTemplateLoader(FormJavaGenerator.class,"/formgenerator/"));
        c.setDefaultEncoding("UTF-8");
        c.setTemplateExceptionHandler(TemplateExceptionHandler.RETHROW_HANDLER);
        c.setLogTemplateExceptions(false);
        c.setWrapUncheckedExceptions(true);
        c.setFallbackOnNullLoopVariable(false);
        c.setWhitespaceStripping(true);
        c.setNumberFormat("0.##############");
        return c;
    }

    protected JavaCompiler initJavaCompiler() {
        JavaCompiler compiler = ToolProvider.getSystemJavaCompiler();
        return compiler;
    }

    public Configuration getConfiguration() {
        return this.configuration;
    }

    public JavaCompiler getJavaCompiler() {
        return this.javaCompiler;
    }

    public DataSource getDataSource() {
        return null;
    }


    /*** 工具接口 **/

    /**
     * 用FreeMarker根据模版生成java代码
     * @param model 数据模型
     * @param jtl 模版名称
     * @return
     */
    public String doGenerateJavaCodeByFM(Object model,String jtl) {
        String r = null;
        Configuration c = getConfiguration();
        try{
            Template t = c.getTemplate(jtl);
            StringWriter w = new StringWriter();
            t.process(model,w);
            r = w.getBuffer().toString();
        } catch (Exception e) {
            log.error("表单引擎：生成java代码错误，e="+e.getMessage(),e);
            throw new AppException(e);
        }
        return r;
    }

    public void doCompilerJavaClassByMemory(String fullClassName,String javaCode) {
        JavaCompiler compiler = getJavaCompiler();
        DiagnosticCollector<JavaFileObject> diagnostics = new DiagnosticCollector<>();
        MemoryFileManager manager = new MemoryFileManager(compiler.getStandardFileManager(null, null, null));
        List<JavaFileObject> sourceFiles = Collections.singletonList(new MemoryJavaFileObject(fullClassName, javaCode));

        JavaCompiler.CompilationTask task = compiler.getTask(null, manager, diagnostics, null, null, sourceFiles);

        boolean result = task.call();
        if(result) {
            log.info("表单引擎："+fullClassName + " 编译成功");
        } else {
            List<String> ls = CollectionUtils.convert(diagnostics.getDiagnostics(),Diagnostic::toString);
            String msg = "表单引擎："+fullClassName + "编译失败";
            log.error(msg + CommonConst.LF + CollectionUtil.join(ls,String.valueOf(CommonConst.LF)));
            throw new AppException(msg);
        }
    }

    public void doLoadJavaClassByMemory(String fullClassName) {
        try{
//            ClassLoader classLoader = manager.getClassLoader(null);
            ClassLoader classLoader = null;
            Class<?> clazz = classLoader.loadClass(fullClassName);
//            IFormBean bean = (IFormBean) clazz.getConstructor().newInstance();
//            bean.setName("aaaaa");
//            log.info("表单输出name："+bean.getName());
        } catch (Exception e) {
            throw new AppException("表单引擎：加载字节码失败"+ fullClassName,e);
        }
    }

    public void doCache(String key, Object value) {
        Object old = this.cache.put(key, value);
        if(old != null) {
            log.warn("表单引擎：缓存被覆盖，key="+key);
        }
    }

    public Object getCache(String key) {
        return this.cache.get(key);
    }

}
