package com.kingtom.kirin.app.org.service;

import java.util.List;
import java.util.stream.Collectors;

import com.github.pagehelper.PageInfo;
import com.kingtom.kirin.app.org.model.OrgPerson;
import com.kingtom.kirin.app.org.model.OrgPersonExample;
import org.springframework.transaction.annotation.Transactional;

/**
 * 组织模块</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023 /10/14
 */
public interface IOrgPersonService{

    /**
     * 插入一条记录（选择字段，策略插入）
     *
     * @param entity 实体对象
     * @return org person
     */
    OrgPerson createPerson(OrgPerson entity);


    /**
     * 插入（批量）
     *
     * @param entityList 实体对象集合
     */
    @Transactional(rollbackFor = Exception.class)
    default void createPersonBatch(List<OrgPerson> entityList){
        for(OrgPerson entity : entityList){
            createPerson(entity);
        }
    }

    /**
     * 根据 ID 删除
     *
     * @param id 主键ID
     * @return the boolean
     */
    boolean deletePersonById(String id);

    /**
     * 根据 entity 条件，删除记录
     *
     * @param example 实体包装类 {@link com.kingtom.kirin.app.org.model.OrgPersonExample}
     * @return the boolean
     */
    boolean deletePerson(OrgPersonExample example);

    /**
     * 删除（根据ID 批量删除）
     *
     * @param idList 主键ID列表
     * @return the boolean
     */
    boolean deletePersonByIds(List<String> idList);

    /**
     * 根据 ID 选择修改
     *
     * @param entity 实体对象
     * @return org person
     */
    OrgPerson updatePersonById(OrgPerson entity);

    /**
     * 根据 whereEntity 条件，更新记录
     *
     * @param entity  实体对象
     * @param example 实体包装类 {@link com.kingtom.kirin.app.org.model.OrgPersonExample}
     * @return list list
     */
    List<OrgPerson> updatePerson(OrgPerson entity, OrgPersonExample example);

    /**
     * 根据ID 批量更新
     *
     * @param entityList 实体对象集合
     * @return list list
     */
    @Transactional(rollbackFor = Exception.class)
    default List<OrgPerson> updatePersonBatchByIds(List<OrgPerson> entityList){
        return entityList.stream().map(this::updatePersonById).collect(Collectors.toList());
    }

    /**
     * 根据 ID 查询
     *
     * @param id 主键ID
     * @return the org person
     */
    OrgPerson findPersonById(String id);

    /**
     * 根据 手机号 查询
     *
     * @param phone the phone
     * @return the org person
     */
    OrgPerson findPersonByPhone(String phone);

    /**
     * 查询（根据ID 批量查询）
     *
     * @param idList 主键ID列表
     * @return the list
     */
    List<OrgPerson> findPersonListByIds(List<String> idList);

    /**
     * 根据 Wrapper，查询一条记录 <br/>
     * <p>结果集，如果是多个会抛出异常</p>
     *
     * @param example 实体包装类 {@link com.kingtom.kirin.app.org.model.OrgPersonExample}
     * @return the org person
     */
    default OrgPerson findPersonOne(OrgPersonExample example){
        return findPersonOne(example, true);
    }

    /**
     * 根据 Wrapper，查询一条记录
     *
     * @param example 实体包装类 {@link com.kingtom.kirin.app.org.model.OrgPersonExample}
     * @param throwEx 有多个 result 是否抛出异常
     * @return the org person
     */
    OrgPerson findPersonOne(OrgPersonExample example, boolean throwEx);

    /**
     * 根据 Wrapper 条件，查询总记录数
     *
     * @param example 实体包装类 {@link com.kingtom.kirin.app.org.model.OrgPersonExample}
     * @return the long
     */
    long findPersonCount(OrgPersonExample example);


    /**
     * 查询列表
     *
     * @param example 实体包装类 {@link com.kingtom.kirin.app.org.model.OrgPersonExample}
     * @return the list
     */
    List<OrgPerson> findPersonList(OrgPersonExample example);

    /**
     * 翻页查询
     *
     * @param example  实体包装类 {@link com.kingtom.kirin.app.org.model.OrgPersonExample}
     * @param pageNum  当前页
     * @param pageSize 页面数据条数
     * @return page info
     */
    PageInfo<OrgPerson> findPersonPage(OrgPersonExample example, int pageNum, int pageSize);


    /**
     * Find person page by struct page info.
     *
     * @param code        the code
     * @param search      the search
     * @param name        the name
     * @param phone
     * @param licenseCode the license no
     * @param pageNum     the page num
     * @param pageSize    the page size
     * @return the page info
     */
    PageInfo<OrgPerson> findPersonPageByStruct(String code, String search, String name, String phone,
            String licenseCode, int pageNum, int pageSize);

}
