package com.kingtom.kirin.app.org.service;

import java.util.List;
import java.util.stream.Collectors;

import com.github.pagehelper.PageInfo;
import com.kingtom.kirin.app.org.OrgConst;
import com.kingtom.kirin.app.org.model.OrgStruct;
import com.kingtom.kirin.app.org.model.OrgStructExample;
import org.springframework.transaction.annotation.Transactional;

/**
 * 应用模块名称</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023 /10/18
 */
public interface IOrgStructService{

    /**
     * 插入一条记录（选择字段，策略插入）
     *
     * @param entity 实体对象
     * @return org struct
     */
    OrgStruct createStruct(OrgStruct entity);


    /**
     * 插入（批量）
     *
     * @param entityList 实体对象集合
     */
    @Transactional(rollbackFor = Exception.class)
    default void createStructBatch(List<OrgStruct> entityList){
        for(OrgStruct entity : entityList){
            createStruct(entity);
        }
    }

    /**
     * 根据 ID 删除
     *
     * @param id 主键ID
     * @return the boolean
     */
    boolean deleteStructById(String id);

    /**
     * Delete struct and child boolean.
     *
     * @param id the id
     * @return the boolean
     */
    boolean deleteStructAndChild(String id);


    /**
     * 根据 entity 条件，删除记录
     *
     * @param example 实体包装类 {@link com.kingtom.kirin.app.org.model.OrgStructExample}
     * @return the boolean
     */
    boolean deleteStruct(OrgStructExample example);

    /**
     * 删除（根据ID 批量删除）
     *
     * @param idList 主键ID列表
     * @return the boolean
     */
    boolean deleteStructByIds(List<String> idList);

    /**
     * 根据 ID 选择修改
     *
     * @param entity 实体对象
     * @return org struct
     */
    OrgStruct updateStructById(OrgStruct entity);

    /**
     * 根据 whereEntity 条件，更新记录
     *
     * @param entity  实体对象
     * @param example 实体包装类 {@link com.kingtom.kirin.app.org.model.OrgStructExample}
     * @return list list
     */
    List<OrgStruct> updateStruct(OrgStruct entity, OrgStructExample example);

    /**
     * 根据ID 批量更新
     *
     * @param entityList 实体对象集合
     * @return list list
     */
    @Transactional(rollbackFor = Exception.class)
    default List<OrgStruct> updateStructBatchByIds(List<OrgStruct> entityList){
        return entityList.stream().map(this::updateStructById).collect(Collectors.toList());
    }

    /**
     * 根据 ID 查询
     *
     * @param id 主键ID
     * @return the org struct
     */
    OrgStruct findStructById(String id);

    /**
     * Find struct by code and org id org struct.
     *
     * @param code  the code
     * @param orgId the org id
     * @return org struct
     */
    OrgStruct findStructByCodeAndOrgId(String code, String orgId);

    OrgStruct findStructByFullName(String fullName, String orgId);

    /**
     * find
     *
     * @param subTypeEnum the sub type enum
     * @param orgId       the org id
     * @return list list
     */
    List<OrgStruct> findStructByOrgId(OrgConst.OrgStructSubTypeEnum subTypeEnum, String orgId);

    /**
     * 查询（根据ID 批量查询）
     *
     * @param idList 主键ID列表
     * @return the list
     */
    List<OrgStruct> findStructListByIds(List<String> idList);

    /**
     * 根据 Wrapper，查询一条记录 <br/>
     * <p>结果集，如果是多个会抛出异常</p>
     *
     * @param example 实体包装类 {@link com.kingtom.kirin.app.org.model.OrgStructExample}
     * @return the org struct
     */
    default OrgStruct findStructOne(OrgStructExample example){
        return findStructOne(example, true);
    }

    /**
     * 根据 Wrapper，查询一条记录
     *
     * @param example 实体包装类 {@link com.kingtom.kirin.app.org.model.OrgStructExample}
     * @param throwEx 有多个 result 是否抛出异常
     * @return the org struct
     */
    OrgStruct findStructOne(OrgStructExample example, boolean throwEx);

    /**
     * 根据 Wrapper 条件，查询总记录数
     *
     * @param example 实体包装类 {@link com.kingtom.kirin.app.org.model.OrgStructExample}
     * @return the long
     */
    long findStructCount(OrgStructExample example);


    /**
     * 查询列表
     *
     * @param example 实体包装类 {@link com.kingtom.kirin.app.org.model.OrgStructExample}
     * @return the list
     */
    List<OrgStruct> findStructList(OrgStructExample example);

    /**
     * 翻页查询
     *
     * @param example  实体包装类 {@link com.kingtom.kirin.app.org.model.OrgStructExample}
     * @param pageNum  当前页
     * @param pageSize 页面数据条数
     * @return page info
     */
    PageInfo<OrgStruct> findStructPage(OrgStructExample example, int pageNum, int pageSize);

    /**
     * Find org struct page info.
     *
     * @param query    the query
     * @param name     the name
     * @param usci     the usci
     * @param pageNum  the page num
     * @param pageSize the page size
     * @return the page info
     */
    PageInfo<OrgStruct> findOrgStructPage(String query, String name, String usci, Integer pageNum, Integer pageSize);


    /**
     * Find struct and child list.
     *
     * @param id the id
     * @return the list
     */
    List<OrgStruct> findStructAndChild(String id);
}
