package com.kingtom.kirin.app.org.service;

import java.util.List;
import java.util.stream.Collectors;

import com.github.pagehelper.PageInfo;
import com.kingtom.kirin.app.org.model.OrgUnit;
import com.kingtom.kirin.app.org.model.OrgUnitExample;
import org.springframework.transaction.annotation.Transactional;

/**
 * 组织模块</p>
 * 代码描述</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023 /10/16
 */
public interface IOrgUnitService{

    /**
     * 插入一条记录（选择字段，策略插入）
     *
     * @param entity 实体对象
     * @return org unit
     */
    OrgUnit createUnit(OrgUnit entity);


    /**
     * 插入（批量）
     *
     * @param entityList 实体对象集合
     */
    @Transactional(rollbackFor = Exception.class)
    default void createUnitBatch(List<OrgUnit> entityList){
        for(OrgUnit entity : entityList){
            createUnit(entity);
        }
    }

    /**
     * 根据 ID 删除
     *
     * @param id 主键ID
     * @return the boolean
     */
    boolean deleteUnitById(String id);

    /**
     * 根据 entity 条件，删除记录
     *
     * @param example 实体包装类 {@link com.kingtom.kirin.app.org.model.OrgUnitExample}
     * @return the boolean
     */
    boolean deleteUnit(OrgUnitExample example);

    /**
     * 删除（根据ID 批量删除）
     *
     * @param idList 主键ID列表
     * @return the boolean
     */
    boolean deleteUnitByIds(List<String> idList);

    /**
     * 根据 ID 选择修改
     *
     * @param entity 实体对象
     * @return org unit
     */
    OrgUnit updateUnitById(OrgUnit entity);

    /**
     * 根据 whereEntity 条件，更新记录
     *
     * @param entity  实体对象
     * @param example 实体包装类 {@link com.kingtom.kirin.app.org.model.OrgUnitExample}
     * @return list
     */
    List<OrgUnit> updateUnit(OrgUnit entity, OrgUnitExample example);

    /**
     * 根据ID 批量更新
     *
     * @param entityList 实体对象集合
     * @return list
     */
    @Transactional(rollbackFor = Exception.class)
    default List<OrgUnit> updateUnitBatchByIds(List<OrgUnit> entityList){
        return entityList.stream().map(this::updateUnitById).collect(Collectors.toList());
    }

    /**
     * 根据 ID 查询
     *
     * @param id 主键ID
     * @return the org unit
     */
    OrgUnit findUnitById(String id);

    /**
     * 根据 手机号 查询
     *
     * @param usci the usci
     * @return the org unit
     */
    OrgUnit findUnitByUsci(String usci);

    /**
     * 查询（根据ID 批量查询）
     *
     * @param idList 主键ID列表
     * @return the list
     */
    List<OrgUnit> findUnitListByIds(List<String> idList);

    /**
     * 根据 Wrapper，查询一条记录 <br/>
     * <p>结果集，如果是多个会抛出异常</p>
     *
     * @param example 实体包装类 {@link com.kingtom.kirin.app.org.model.OrgUnitExample}
     * @return the org unit
     */
    default OrgUnit findUnitOne(OrgUnitExample example){
        return findUnitOne(example, true);
    }

    /**
     * 根据 Wrapper，查询一条记录
     *
     * @param example 实体包装类 {@link com.kingtom.kirin.app.org.model.OrgUnitExample}
     * @param throwEx 有多个 result 是否抛出异常
     * @return the org unit
     */
    OrgUnit findUnitOne(OrgUnitExample example, boolean throwEx);

    /**
     * 根据 Wrapper 条件，查询总记录数
     *
     * @param example 实体包装类 {@link com.kingtom.kirin.app.org.model.OrgUnitExample}
     * @return the long
     */
    long findUnitCount(OrgUnitExample example);


    /**
     * 查询列表
     *
     * @param example 实体包装类 {@link com.kingtom.kirin.app.org.model.OrgUnitExample}
     * @return the list
     */
    List<OrgUnit> findUnitList(OrgUnitExample example);

    /**
     * 翻页查询
     *
     * @param example  实体包装类 {@link com.kingtom.kirin.app.org.model.OrgUnitExample}
     * @param pageNum  当前页
     * @param pageSize 页面数据条数
     * @return page info
     */
    PageInfo<OrgUnit> findUnitPage(OrgUnitExample example, int pageNum, int pageSize);

    /**
     * Find org unit page info.
     *
     * @param query    the query
     * @param name     the name
     * @param usci     the usci
     * @param pageNum  the page num
     * @param pageSize the page size
     * @return the page info
     */
    PageInfo<OrgUnit> findOrgUnitPage(String query, String name, String usci, Integer pageNum, Integer pageSize);

}
