package com.kingtom.kirin.app.security.service.impl;

import com.kingtom.kirin.app.security.base.IAuthcRealm;
import com.kingtom.kirin.app.security.base.IPermissionLoader;
import com.kingtom.kirin.app.security.base.ISessionDataLoader;
import com.kingtom.kirin.app.security.service.ISecurityContainerService;
import com.kingtom.kirin.app.system.SystemConst;
import com.kingtom.kirin.core.common.exception.AppException;
import org.springframework.context.annotation.Scope;
import org.springframework.context.annotation.ScopedProxyMode;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.function.Consumer;

/**
 * 会话安全模块</p>
 * 用于存储管理模块内的组件的容器，不对外开放</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/8/31
 */
@Component
@Scope(proxyMode = ScopedProxyMode.NO)
public class SecurityContainerServiceImpl implements ISecurityContainerService {

    /**
     * 登录认证方式
     */
    private Map<SystemConst.AuthcRealmTypeEnum, IAuthcRealm> realmMap = new ConcurrentHashMap<>();

    public void registerRealm(IAuthcRealm realm){
        if(realm == null){
            throw new AppException();
        }
        realmMap.put(realm.support(), realm);
    }

    public IAuthcRealm getRealm(SystemConst.AuthcRealmTypeEnum key){
        return realmMap.get(key);
    }

    /**
     * 会话信息加载器
     */
    private final List<ISessionDataLoader> sessionLoaderContainer = new ArrayList<>();

    /**
     * 注册安全监听
     *
     * @param loader the loader
     */
    public void registerSessionLoader(ISessionDataLoader loader){
        if(loader == null){
            throw new AppException("安全监听不能为空");
        }
        if(sessionLoaderContainer.contains(loader)){
            throw new AppException("安全监听不能重复");
        }
        int order = loader.getOrder();
        if(sessionLoaderContainer.isEmpty()){
            sessionLoaderContainer.add(loader);
        } else{
            int insert = 0;
            for(int i = (sessionLoaderContainer.size() - 1); i >= 0; i--){
                int o = sessionLoaderContainer.get(i).getOrder();
                if(o <= order){
                    insert = i + 1;
                    break;
                }
            }
            sessionLoaderContainer.add(insert, loader);
        }
        sessionLoaderContainer.sort(Comparator.comparingInt(ISessionDataLoader::getOrder));
    }

    public void foreachSessionLoader(Consumer<ISessionDataLoader> action){
        sessionLoaderContainer.forEach(action);
    }

    /**
     * 权限解释器
     */
    private final List<IPermissionLoader> permissionLoaderContainer = new ArrayList<>();

    public void registerPermissionLoader(IPermissionLoader loader){
        if(loader == null){
            throw new AppException("安全监听不能为空");
        }
        permissionLoaderContainer.add(loader);
    }

    public void foreachPermissionLoader(Consumer<IPermissionLoader> function){
        permissionLoaderContainer.forEach(function);
    }

}
