package com.kingtom.kirin.app.security.service.impl;

import java.util.*;
import java.util.stream.Collectors;

import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.kingtom.kirin.app.auth.model.AuthPermission;
import com.kingtom.kirin.app.security.base.IAuthcRealm;
import com.kingtom.kirin.app.security.base.SessionData;
import com.kingtom.kirin.app.security.service.ISecurityContainerService;
import com.kingtom.kirin.app.security.service.ISecurityService;
import com.kingtom.kirin.app.system.SystemConst;
import com.kingtom.kirin.app.system.model.SystemUser;
import com.kingtom.kirin.app.system.model.SystemUserExt3;
import com.kingtom.kirin.app.system.model.UserSecurity;
import com.kingtom.kirin.app.system.service.ISystemService;
import com.kingtom.kirin.app.system.service.IUserSecurityService;
import com.kingtom.kirin.app.system.utils.SystemUtils;
import com.kingtom.kirin.core.common.exception.AppException;
import com.kingtom.kirin.core.common.json.JObject;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.common.utils.ObjectUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/11/6 15:42
 */
@Component
@Slf4j
public class SecurityServiceImpl implements ISecurityService {

    @Autowired
    private ISecurityContainerService container;

    @Autowired
    private IUserSecurityService userSecurityService;

    @Autowired
    private ISystemService systemService;

    @Autowired
    private ISecurityContainerService ISecurityContainerService;

    /**
     * 用户数据map：key(token)->value(sessionData)
     */
    private final Map<String, SessionData> sessionDataMap = MapUtil.newConcurrentHashMap();

    /**
     * 用户user-tokenMap：key(userId)->value(token集合)
     */
    private final Map<String, Set<String>> userTokenMap = MapUtil.newConcurrentHashMap();

    @Override
    public SessionData logIn(String token, String userId){
        if(ObjectUtils.isAnyNull(token)) {
            throw new AppException("未登录");
        } else {
            SessionData dto = sessionDataMap.computeIfAbsent(token, k->{
                return SessionData.build(k, userId, SystemUtils.getCurrentSystemId());
            });
            if(!dto.getUserId().equals(userId)) {
                log.info("登录用户对应发生了切换，oldUserId="+dto.getUserId()+";newUserId="+userId);
                unbindUserToken(dto.getUserId(),token);
                SystemUser user = systemService.findSystemUserById(userId);
                dto.changeUser(user.getId(),user.getResAttr().getE().getPhone(),user.getSysId());
            }
            bindUserToken(dto.getUserId(),dto.getToken());
            doLogin("登录", dto);
            return dto;
        }
    }

    @Override
    public SessionData logOut(String token){
        if(ObjectUtils.isAnyNull(token)) {
            throw new AppException("未登录");
        } else {
            SessionData dto = sessionDataMap.remove(token);
            doLogout("退出", dto);
            return dto;
        }
    }

    @Override
    public SessionData findSessionData(String token) {
        if(StrUtil.isBlank(token)) {
            throw new AppException("未登录");
        } else {
            SessionData r = sessionDataMap.get(token);
            doLoad("获取用户数据",r);
            return r;
        }
    }

    @Override
    public SessionData findSessionDataByUserId(String userId) {
        String token = findFirstToken(userId);
        return findSessionData(token);
    }

    @Override
    public List<String> parsePermissions(List<AuthPermission> permissions){
        Set<String> s = new HashSet<>();
        container.foreachPermissionLoader(l -> {
            List<AuthPermission> list = permissions.stream().filter(e -> l.define().getKey().equals(e.getResType()))
                    .collect(Collectors.toList());
            List<String> codes = l.parsePermission(list);
            if(codes != null){
                s.addAll(codes);
            }
        });
        return new ArrayList<>(s);
    }

    @Override
    public void updateCredentials(String userId, String oldPassword, String password){
        IAuthcRealm realm = container.getRealm(SystemConst.AuthcRealmTypeEnum.Password);
        List<UserSecurity> list = userSecurityService.findByUserId(realm.support().getTypeEnums(), userId);
        //密码的加密和校验
        UserSecurity userSecurity = list.stream().filter(e -> {
            SystemConst.AuthcEncodeEnum enumByKey = SystemConst.AuthcEncodeEnum.findEnumByKey(e.getEncode());
            return e.getCredentials().equals(enumByKey.encode(oldPassword));
        }).findFirst().orElse(null);
        if(userSecurity == null){
            throw new AppException("历史密码错误！");
        }
        for(UserSecurity security : list){
            UserSecurity updateS = new UserSecurity();
            updateS.setId(security.getId());
            updateS.setCredentials(SystemConst.AuthcEncodeEnum.Md5.encode(password));
            updateS.setEncode(SystemConst.AuthcEncodeEnum.Md5.getKey());
            userSecurityService.updateUserSecurity(updateS);
        }

        //主动调用用户信息更新
        SystemUser systemUser = systemService.findSystemUserById(userId);
        if(systemUser.getExt3() == null){
            systemUser.setExt3(new JObject<>());
        }
        if(systemUser.getExt3().getE() == null){
            systemUser.getExt3().setE(new SystemUserExt3());
        }
        systemUser.getExt3().getE().setPassword(password);
        systemService.updateSystemUser(systemUser);
    }

    @Override
    public List<UserSecurity> findByPrincipal(List<SystemConst.UserSecurityTypeEnum> authTypeEnums, String principal){
        return userSecurityService.findByPrincipal(authTypeEnums, principal);
    }

    protected void doLoad(String msg, SessionData dto) {
        if(dto == null) {
            log.warn("获取用户信息无法执行,"+msg);
        } else {
            ISecurityContainerService.foreachSessionLoader(l->l.onLoad(dto));
            //log.info("获取用户信息完成,"+msg);
        }
    }

    protected void doLogin(String msg, SessionData dto) {
        if(dto == null) {
            log.warn("用户登录监听无法执行,"+msg);
        } else {
            ISecurityContainerService.foreachSessionLoader(l->l.onLogin(dto));
            log.info("用户登录完成,"+msg);
        }
    }

    protected void doLogout(String msg, SessionData dto) {
        if(dto == null) {
            log.warn("用户登出监听无法执行,"+msg);
        } else {
            ISecurityContainerService.foreachSessionLoader(l->l.onLogout(dto));
            log.info("用户登出完成,"+msg);
        }
    }

    protected void unbindUserToken(String userId,String token) {
        Set<String> tokens = userTokenMap.get(userId);
        if(CollectionUtils.isNotEmpty(tokens)) {
            tokens.remove(token);
        }
        if(CollectionUtils.isEmpty(tokens)) {
            userTokenMap.remove(userId);
        }
    }

    protected void bindUserToken(String userId,String token) {
        Set<String> userTokenSet = userTokenMap.computeIfAbsent(userId, k->new LinkedHashSet<>());
        userTokenSet.add(token);
    }

    protected String findFirstToken(String userId) {
        Set<String> tokens = userTokenMap.get(userId);
        return CollectionUtils.findFirst(tokens);
    }
}
