package com.kingtom.kirin.app.space.service;

import cn.hutool.core.img.ImgUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.util.ObjectUtil;
import com.kingtom.kirin.app.space.SpaceConst;
import com.kingtom.kirin.app.space.base.IFileSource;
import com.kingtom.kirin.app.space.base.SpaceFileReadFileSource;
import com.kingtom.kirin.app.space.model.SpaceFile;
import com.kingtom.kirin.app.space.model.StorageArea;
import com.kingtom.kirin.app.space.utils.SpaceUtils;
import com.kingtom.kirin.core.common.exception.AppException;
import com.kingtom.kirin.core.common.utils.JsonUtils;
import jakarta.annotation.PostConstruct;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 * 专门负责存储
 * @author King
 * @since 2023/6/13 11:40
 */
@Component
@Slf4j
public class StorageService {

    @Value("${app.storage.path}")
    private String storagePathStr;
    @Value("${app.storage.nginx}")
    private String storageNginx;

    private String storageImageScale = "0.5f";
    private StorageArea defaultStorageArea;

    @PostConstruct
    protected void init(){
        defaultStorageArea = initStorageAreaDefault(storagePathStr,storageNginx);
        if(defaultStorageArea == null) {
            throw new AppException("初始化存储空间失败，path=" + storagePathStr);
        } else {
            log.info("初始化存储空间完成，area="+ JsonUtils.toJson(defaultStorageArea));
        }
    }

    public void doStorage(SpaceFile file, IFileSource source) {
        File f = defaultStorageArea.findFormatFile(file, true);
        OutputStream fos = null;
        InputStream is = null;
        try{
            fos = new FileOutputStream(f);
            is = source.getInputStream();
            long size = IoUtil.copy(is, fos);
            file.setSize(size);
            file.setSaveSize(size);
            file.setSaveName(defaultStorageArea.findFormatName(file));
            file.setStorageArea(defaultStorageArea.getId());
            file.setStorageEncode(SpaceConst.SpaceFileStorageEncodeEnum.No.getKey());
        } catch (Exception e) {
            throw new AppException("保存文件失败",e).setM(JsonUtils.toJson(file));
        } finally {
            IoUtil.close(is);
            IoUtil.close(fos);
        }
    }

    /**
     * 做图片的压缩
     * @param file
     */
    public void doImageScaleStorage(SpaceFile file) {
        File f = defaultStorageArea.findFormatFile(file, false);
        if(FileUtil.isEmpty(f)) {
            throw new AppException("文件不存在，无法缩略");
        }
        if(SpaceUtils.isImage(file)) {
            if(SpaceUtils.isNotScaled(file)) {
                File df = defaultStorageArea.findFormatFileOfDerivative(file, storageImageScale, true);
                ImgUtil.scale(f, df, Float.valueOf(storageImageScale));
                file.setExt1(df.getName());
            } else {
                log.info("图片已经被压缩过,id="+file.getId()+";scaleName="+file.getExt1());
            }
        } else {
            log.warn("文件不是支持压缩的图片,id="+file.getId()+";type="+file.getFileType());
        }
    }

    public void deleteStorage(SpaceFile file) {
        File f = defaultStorageArea.findFormatFile(file, false);
        if(FileUtil.isNotEmpty(f)) {
            f.delete();
        }
        if(SpaceUtils.isImage(file)) {
            f = defaultStorageArea.findFormatFileOfDerivative(file, storageImageScale,false);
            if(FileUtil.isNotEmpty(f)) {
                f.delete();
            }
        }
    }

    public IFileSource readStorage(SpaceFile file) {
        File f = defaultStorageArea.findFormatFile(file, false);
        if(FileUtil.isNotEmpty(f)) {
            if(SpaceUtils.isImage(file)) {
                String httpUrl = defaultStorageArea.findFormatHttpUrl(file);
                File df = defaultStorageArea.findFormatFileOfDerivative(file,storageImageScale, false);
                String dhttpUrl = FileUtil.isNotEmpty(df)?defaultStorageArea.findFormatHttpUrlOfDerivative(file,storageImageScale):null;
                return new SpaceFileReadFileSource(file,f, httpUrl,dhttpUrl);
            } else {
                return new SpaceFileReadFileSource(file,f);
            }

        } else {
            throw new AppException("文件不存在").setM(JsonUtils.toJson(file));
        }
    }

    protected static StorageArea initStorageAreaDefault(String storagePathStr,String storageNginx) {
        StorageArea r = new StorageArea();
        r.setId(SpaceConst.STORAGE_AREA_DEFAULT_ID);
        r.setAreaName(SpaceConst.STORAGE_AREA_DEFAULT_NAME);
        r.setNginx(storageNginx);
        // 标准存储目录
        File f = new File(storagePathStr);
        if(f.exists() || f.mkdirs()) {
            r.setPath(f);
            return r;
        } else {
            return null;
        }
    }

//    public static void main(String[] args) {
//        String path = "D:\\work\\storage";
//        StorageArea area = initStorageAreaDefault(path);
//        SpaceFile spaceFile = new SpaceFile();
//        spaceFile.setId("a11111-22222");
//        spaceFile.setGmtCreate(new Date());
//        File sf = findStorageFile(area, spaceFile);
//        System.out.println(sf.exists());
//    }

}
