package com.kingtom.kirin.app.system.service.impl;

import java.util.Date;
import java.util.List;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import com.kingtom.kirin.app.system.SystemConst;
import com.kingtom.kirin.app.system.dao.UserSecurityMapper;
import com.kingtom.kirin.app.system.model.UserSecurity;
import com.kingtom.kirin.app.system.model.UserSecurityExample;
import com.kingtom.kirin.app.system.service.IUserSecurityService;
import com.kingtom.kirin.app.system.utils.SystemUtils;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.exception.AppException;
import com.kingtom.kirin.core.common.utils.CollectionUtils;
import com.kingtom.kirin.core.common.utils.IDUtils;
import com.kingtom.kirin.core.common.utils.JsonUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 权限模块</p>
 * 安全凭证 pojo service impl</p>
 * Copyright: Copyright (C) 2023 , Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限责任公司</p>
 *
 * @author wany
 * @since 2023/8/24
 */
@Service
@Slf4j
public class UserSecurityServiceImpl implements IUserSecurityService{

    /**
     * 由8~16位字符（英文字母、数字、非空白字符）组成,
     */
    private static final Pattern pattern = Pattern.compile("^[\\S]{8,16}$");

    @Autowired
    private UserSecurityMapper mapper;

    @Override
    public UserSecurity createUserSecurity(UserSecurity security){
        UserSecurity db = findByPrincipal(SystemConst.UserSecurityTypeEnum.findEnumByKey(security.getType()),
                security.getPrincipal());
        if(db != null){
            // 如果这个凭证存在就先删除
//            throw new AppException("凭证已存在，新建登录凭证失败！");
            log.warn("凭证已存在，准备覆盖该凭证, db="+ JsonUtils.toJson(db)+";security="+JsonUtils.toJson(security));
            mapper.deleteByPrimaryKey(db.getId());
        }
//        if(!pattern.matcher(security.getCredentials()).matches()){
//            throw new AppException("登录凭证密码格式错误！");
//        }
        security.setId(IDUtils.newUUID());
        security.setGmtCreate(new Date());
        security.setGmtModified(security.getGmtCreate());
        security.setSysId(SystemUtils.getCurrentSystemId());
        boolean f = mapper.insertSelective(security) == 1;
        if(!f){
            throw new AppException("落库出错，新建登录凭证失败");
        }
        return findUserSecurity(security.getId());
    }

    @Override
    public UserSecurity updateUserSecurity(UserSecurity security){
        if(security == null || StringUtils.isBlank(security.getId())){
            throw new AppException("凭证信息错误，更新登录凭证失败！");
        }
        UserSecurity byId = findUserSecurity(security.getId());
        if(byId == null){
            throw new AppException("凭证不存在，更新登录凭证失败！");
        }
        if(StringUtils.isNoneBlank(security.getType(), security.getPrincipal())){
            SystemConst.UserSecurityTypeEnum authTypeEnum =
                    SystemConst.UserSecurityTypeEnum.findEnumByKey(security.getType());
            if(authTypeEnum == null){
                throw new AppException("凭证信息错误,更新登录凭证失败！");
            }
            UserSecurity db = findByPrincipal(authTypeEnum, security.getPrincipal());
            if(db != null && !db.getId().equals(byId.getId())){
                throw new AppException("凭证已存在,更新登录凭证失败！");
            }
        }
//        if(security.getCredentials() != null && !pattern.matcher(security.getCredentials()).matches()){
//            throw new AppException("登录凭证密码格式错误！");
//        }
        UserSecurity update = new UserSecurity();
        update.setGmtModified(new Date());
        update.setId(security.getId());
        if(StringUtils.isNotBlank(security.getUserId())){
            update.setUserId(security.getUserId());
        }
        if(StringUtils.isNotBlank(security.getPrincipal())){
            update.setPrincipal(security.getPrincipal());
        }
        if(StringUtils.isNotBlank(security.getCredentials())){
            update.setCredentials(security.getCredentials());
            update.setEncode(security.getEncode());
        }
        boolean f = mapper.updateByPrimaryKeySelective(update) == 1;
        if(!f){
            throw new AppException("落库出错，更新登录凭证失败！");
        }
        return findUserSecurity(security.getId());
    }

    @Override
    public boolean deleteUserSecurity(String securityId){
        return mapper.deleteByPrimaryKey(securityId) == 1;
    }

    @Override
    public UserSecurity findUserSecurity(String securityId){
        return mapper.selectByPrimaryKey(securityId);
    }

    @Override
    public UserSecurity findUserSecurityByType(String userId, SystemConst.UserSecurityTypeEnum support) {
        UserSecurityExample example = new UserSecurityExample();
        UserSecurityExample.Criteria criteria = example.createCriteria();
        criteria.andUserIdEqualTo(userId).andTypeEqualTo(support.getKey());
        List<UserSecurity> userSecurityList = mapper.selectByExample(example);
        return CollectionUtils.findFirst(userSecurityList);
    }

    @Override
    public UserSecurity findByPrincipal(String principal){
        UserSecurityExample example = new UserSecurityExample();
        UserSecurityExample.Criteria criteria = example.createCriteria();
        criteria.andPrincipalEqualTo(principal);
        List<UserSecurity> userSecurityList = mapper.selectByExample(example);
        return CollectionUtils.findFirst(userSecurityList);
    }

    @Override
    public UserSecurity findByPrincipal(SystemConst.UserSecurityTypeEnum support, String principal){
        UserSecurityExample example = new UserSecurityExample();
        UserSecurityExample.Criteria criteria = example.createCriteria();
        criteria.andPrincipalEqualTo(principal);
        criteria.andTypeEqualTo(support.getKey());
        List<UserSecurity> userSecurityList = mapper.selectByExample(example);
        return CollectionUtils.isEmpty(userSecurityList) ? null : userSecurityList.get(CommonConst.INT_0);
    }

    @Override
    public List<UserSecurity> findByPrincipal(List<SystemConst.UserSecurityTypeEnum> supports, String principal){
        UserSecurityExample example = new UserSecurityExample();
        UserSecurityExample.Criteria criteria = example.createCriteria();
        criteria.andPrincipalEqualTo(principal);
        criteria.andTypeIn(
                supports.stream().map(SystemConst.UserSecurityTypeEnum::getKey).collect(Collectors.toList()));
        return mapper.selectByExample(example);
    }

    @Override
    public List<UserSecurity> findByUserId(List<SystemConst.UserSecurityTypeEnum> supports, String userId){
        UserSecurityExample example = new UserSecurityExample();
        UserSecurityExample.Criteria criteria = example.createCriteria();
        criteria.andUserIdEqualTo(userId);
        if(CollectionUtils.isNotEmpty(supports)){
            criteria.andTypeIn(
                    supports.stream().map(SystemConst.UserSecurityTypeEnum::getKey).collect(Collectors.toList()));
        }
        return mapper.selectByExample(example);
    }

}
