package com.kingtom.kirin.core.common.utils;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.esotericsoftware.minlog.Log;
import com.kingtom.kirin.core.common.exception.AppException;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;

import java.io.Serializable;
import java.util.List;
import java.util.Map;
import java.util.function.BiFunction;
import java.util.function.Consumer;
import java.util.function.Function;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/9/5 15:22
 */
@Slf4j
public class CaseUtils {
    private CaseUtils(){}

    @SafeVarargs
    public static <T> Context<T> build(T bean, Consumer<Context<T>>... actions) {
        return new Context<T>(bean,ListUtil.toList(actions));
    }

    public static <T> Consumer<Context<T>> caseContextAndDo(Consumer<Context<T>> c) {
        return c;
    }

    @SafeVarargs
    public static <T> Consumer<Context<T>> caseBeanAndThrow(Function<T,Boolean> caseFunc, String expMsg, Function<T,String>... bizMsg) {
        return c -> {
          if(caseFunc.apply(c.getBean())) {
              throw buildAppException(c,expMsg,bizMsg);
          }
        };
    }

    @SafeVarargs
    public static <T> Consumer<Context<T>> caseContextAndThrow(Function<Context<T>,Boolean> caseFunc, String expMsg, Function<T,String>... bizMsg) {
        return c -> {
            if(caseFunc.apply(c)) {
                throw buildAppException(c,expMsg,bizMsg);
            }
        };
    }

    @SafeVarargs
    public static <T> Consumer<Context<T>> caseAttrNullAndThrow(Function<T,Object> attrFunc, String expMsg, Function<T,String>... bizMsg) {
        return c -> {
            if(ObjectUtil.isNull(attrFunc.apply(c.getBean()))) {
                throw buildAppException(c,expMsg,bizMsg);
            }
        };
    }

    @SafeVarargs
    public static <T> Consumer<Context<T>> caseAttrBlankAndThrow(Function<T,String> attrFunc, String expMsg, Function<T,String>... bizMsg) {
        return c -> {
            if(StrUtil.isBlank(attrFunc.apply(c.getBean()))) {
                throw buildAppException(c,expMsg,bizMsg);
            }
        };
    }

    @SafeVarargs
    public static <T> Consumer<Context<T>> caseRelAndThrow(Function<T,Object> relationFunc, Function<Context<T>,Boolean> caseFunc, String expMsg, Function<T,String>... bizMsg) {
        return c -> {
            Object o = relationFunc.apply(c.getBean());
            c.setCacheCurrent(o);
            if(caseFunc.apply(c)) {
                throw buildAppException(c,expMsg,bizMsg);
            }
        };
    }

    @SafeVarargs
    public static <T> Consumer<Context<T>> caseRelCheckAndThrow(Function<T,Object> relationFunc, Function<Object,Boolean> caseFunc, String expMsg, Function<T,String>... bizMsg) {
        return c -> {
            Object o = relationFunc.apply(c.getBean());
            c.setCacheCurrent(o);
            if(caseFunc.apply(o)) {
                throw buildAppException(c,expMsg,bizMsg);
            }
        };
    }

    @SafeVarargs
    public static <T> Consumer<Context<T>> caseRelCompareAndThrow(Function<T,T> relationFunc, BiFunction<T,T, Boolean> caseFunc, String expMsg, Function<T,String>... bizMsg) {
        return c -> {
            T o = relationFunc.apply(c.getBean());
            c.setCacheCurrent(o);
            if(caseFunc.apply(c.getBean(),o)) {
                throw buildAppException(c,expMsg,bizMsg);
            }
        };
    }

    @SafeVarargs
    protected static <T> AppException buildAppException(Context<T> c,String expMsg, Function<T,String>... bizMsg) {
        AppException e = new AppException(expMsg);
        if(bizMsg != null) {
            List<String> msg = ListUtil.toList();
            for(Function<T,String> m:bizMsg) {
                msg.add(m.apply(c.getBean()));
            }
            e.setM(JsonUtils.toJson(msg));
        }
        return e;
    }

    public static class Context<T>{
        private T bean;
        private Map<Integer,Object> cache = MapUtil.newHashMap();
        private List<Consumer<Context<T>>> actions;
        private int step = 0;

        public Context(T bean, List<Consumer<Context<T>>> actions) {
            this.bean = bean;
            this.actions = actions;
        }

        public <C> C getCache(Integer index) {
            return (C)cache.get(index);
        }
        public <C> C getCacheCurrent() {
            return (C)cache.get(Integer.valueOf(step));
        }
        public <C> void setCacheCurrent(C obj) {
            cache.put(Integer.valueOf(step),obj);
        }
        public Context<T> doCase() {
            for(Consumer<Context<T>> action:actions) {
                try{
                    action.accept(this);
                } catch (AppException exception) {
                    exception.setM("组合逻辑第["+step+"]个条件错误："+exception.getM());
                    log.error(exception.getMessage()+":"+exception.getM(),exception);
                    throw exception;
                }
                step++;
            }
            return this;
        }

        public Context<T> clear() {
            this.actions.clear();
            this.cache.clear();
            this.step = 0;
            return this;
        }

        public Context<T> setActions(Consumer<Context<T>>... actions) {
            clear();
            this.actions.addAll(ListUtil.toList(actions));
            return this;
        }

        public T getBean() {
            return bean;
        }
    }

    public static void test1() {
        TestInfo info = new TestInfo();
        info.setId("11");
        CaseUtils.Context<TestInfo> context = CaseUtils.build(info,
                        CaseUtils.caseBeanAndThrow(ObjectUtil::isNull,"配置不能为空"),
                        CaseUtils.caseAttrBlankAndThrow(TestInfo::getId,"ID不能为空"),
                        CaseUtils.caseRelCompareAndThrow(CaseUtils::findOneOfCopy,CaseUtils::isSame,"不能重复"),
                        CaseUtils.caseRelCheckAndThrow(b->findOne(),ObjectUtil::isNull,"不能为空"))
                .doCase();
    }

    public static void test2() {
        TestInfo info = new TestInfo();
        info.setId("11");
        if(ObjectUtil.isNull(info)) {
            throw new AppException("配置不能为空");
        }
        if(StrUtil.isBlank(info.getId())) {
            throw new AppException("ID不能为空");
        }
        if(isSame(findOneOfCopy(info),info)) {
            throw new AppException("不能重复");
        }
        if(ObjectUtil.isNull(findOne())) {
            throw new AppException("不能为空");
        }
    }

    public static void main(String[] args) {
        for(int i=0;i<10;i++){
            try{
                test1();
            } catch (AppException e) {
            }
            try{
                test2();
            } catch (AppException e) {
            }
        }

        long s = System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                test1();
            } catch (AppException e) {
            }
        }
        System.out.println("test1耗时："+(System.currentTimeMillis()-s));

        s = System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                test2();
            } catch (AppException e) {
            }
        }
        System.out.println("test2耗时："+(System.currentTimeMillis()-s));
    }

    public static class TestInfo implements Serializable {
        private String id;
        private String name;

        public String getId() {
            return id;
        }

        public void setId(String id) {
            this.id = id;
        }

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }
    }

    public static TestInfo findOne(){
        return new TestInfo();
    }

    public static TestInfo findOneOfCopy(TestInfo c){
        return ObjectUtils.copyObject(c);
    }

    public static boolean isNotSame(TestInfo o1, TestInfo o2){
        return !o1.equals(o2);
    }

    public static boolean isSame(TestInfo o1, TestInfo o2){
        return o1.equals(o2);
    }

}
