package com.kingtom.kirin.core.common.utils;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.json.JSONObject;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.StreamReadConstraints;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.*;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;
import com.kingtom.kirin.core.common.CommonConst;
import com.kingtom.kirin.core.common.exception.AppException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 * 基于jackson
 * @author King
 * @since 2023/6/4 15:08
 */
@Slf4j
public class JsonUtils {

    /**
     * ObjectMapper本身是线程安全的，不过用local可以略微提高点性能
     */
    private static ThreadLocal<ObjectMapper> jsonMappers = new ThreadLocal<ObjectMapper>();

    private JsonUtils(){}

    private static ObjectMapper getObjectMapper() {
        ObjectMapper r = jsonMappers.get();
        if(r == null) {
            r = new ObjectMapper();
            // 对象的所有字段全部列入，还是其他的选项，可以忽略null等
            r.setSerializationInclusion(JsonInclude.Include.ALWAYS);
            // 设置Date类型的序列化及反序列化格式
            r.setDateFormat(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"));
            // 忽略空Bean转json的错误
            r.configure(SerializationFeature.FAIL_ON_EMPTY_BEANS, false);
            // 忽略未知属性，防止json字符串中存在，java对象中不存在对应属性的情况出现错误
            r.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
            // 注册一个时间序列化及反序列化的处理模块，用于解决jdk8中localDateTime等的序列化问题
            r.registerModule(new JavaTimeModule());
            // 处理base64传输过长的问题
            r.getFactory().setStreamReadConstraints(StreamReadConstraints.builder().maxStringLength(Integer.MAX_VALUE).build());
            jsonMappers.set(r);
        }
        return r;
    }

    /**
     * 对象->JSON
     * @param obj 对象
     * @return json
     */
    public static <T> String toJson(T obj) {
        String r = null;
        if (obj != null) {
            try {
                r = getObjectMapper().writeValueAsString(obj);
            } catch (JsonProcessingException e) {
                log.warn(e.getMessage(), e);
                throw new AppException(e);
            }
        }
        return r;
    }

    public static <T extends JsonNode> T toJsonNode(Object obj) {
        T r = null;
        if (obj != null) {
            r = getObjectMapper().valueToTree(obj);
        }
        return r;
    }

    public static <T extends JsonNode> T toJsonNodeByStr(String json) {
        T r = null;
        if (json != null) {
            try{
                r = (T)getObjectMapper().readTree(json);
            }catch (Exception e) {
                throw new AppException("JSON转对象错误",e);
            }
        }
        return r;
    }

    public static <T> T toObjectByJsonNode(JsonNode node, Class<T> clazz) {
        T r = null;
        if (node != null) {
            r = getObjectMapper().convertValue(node, clazz);
        }
        return r;
    }

    public static <T> T toObject(String json) {
        return toObject(json,null,null,null);
    }

    public static <T> T toObject(String json, TypeReference<T> type) {
        return toObject(json,null,type,null);
    }

    public static <T> T toObject(String json, Class<T> clazz) {
        return toObject(json,clazz,null,null);
    }

    public static <T> List<T> toObjectList(String json, Class<T> clazz) {
        JavaType t = getCollectionType(List.class, clazz);
        return toObject(json,null,null,t);
    }

    /**
     * JSON->对象
     * @param json json
     * @return 对象
     */
    public static <T> T toObject(String json, Class<T> clazz, TypeReference<T> type, JavaType jtype) {
        T r = null;
        if (!StringUtils.isEmpty(json)) {
            ObjectMapper om = getObjectMapper();
            try {
                om.readTree(json);
                if (clazz != null) {
                    r = om.readValue(json, clazz);
                } else if (jtype != null) {
                    r = om.readValue(json, jtype);
                } else {
                    r = om.readValue(json, type);
                }
            } catch (IOException e) {
                log.warn(e.getMessage(), e);
                throw new AppException(e);
            }
        }
        return r;
    }

    public static <T> List<T> toObjectTypeList(List l,List r,Class<T> c) {
        if(l == null) {
            return r;
        }
        r = r == null?ListUtil.toList():r;
        for(Object e:l) {
            if(isJsonObject(e)) {
                r.add(toObjectByJsonNode((ObjectNode)e,c));
            } else {
                r.add((T)e);
            }
        }
        return r;
    }

    public static boolean isJsonObject(Object o) {
        return o != null && o instanceof ObjectNode;
    }

    /**
     * 对象->Map
     * @param obj
     * @return
     * @param <T>
     */
    public static <T> Map<String,?> toMap(T obj) {
        JsonNode json = toJsonNode(obj);
        return toObjectByJsonNode(json, LinkedHashMap.class);
    }

    protected static JavaType getCollectionType(Class<?> collectionClass, Class<?>... elementClasses) {
        return getObjectMapper().getTypeFactory().constructParametricType(collectionClass, elementClasses);
    }

}
